from django.db.models import Sum
from django.views.decorators.cache import never_cache
from multiseek.logic import get_registry
from multiseek.views import (
    MULTISEEK_SESSION_KEY_REMOVED,
    MultiseekResults,
    manually_add_or_remove,
)

from bpp.models import Uczelnia

PKT_WEWN = "pkt_wewn"
PKT_WEWN_BEZ = "pkt_wewn_bez"
TABLE = "table"

EXTRA_TYPES = [
    PKT_WEWN,
    PKT_WEWN_BEZ,
    TABLE,
    PKT_WEWN + "_cytowania",
    PKT_WEWN_BEZ + "_cytowania",
    TABLE + "_cytowania",
]


class MyMultiseekResults(MultiseekResults):
    registry = "bpp.multiseek.registry"

    def get_queryset(self, only_those_ids=None):

        if only_those_ids:
            qset = (
                get_registry(self.registry)
                .get_query_for_model(self.get_multiseek_data())
                .filter(pk__in=only_those_ids)
            )
        else:
            qset = super().get_queryset()

        if not self.request.user.is_authenticated:
            uczelnia = Uczelnia.objects.get_for_request(self.request)
            if uczelnia is not None:
                ukryte_statusy = uczelnia.ukryte_statusy("multiwyszukiwarka")
                if ukryte_statusy:
                    qset = qset.exclude(status_korekty_id__in=ukryte_statusy)

        flds = ("id", "opis_bibliograficzny_cache")

        # wycięte z multiseek/views.py, get_context_data
        report_type = get_registry(self.registry).get_report_type(
            self.get_multiseek_data(), request=self.request
        )

        if report_type in EXTRA_TYPES:
            qset = qset.select_related("charakter_formalny", "typ_kbn")

            flds = flds + (
                "charakter_formalny",
                "typ_kbn",
                "punkty_kbn",
                "impact_factor",
                "adnotacje",
                "uwagi",
                "punktacja_wewnetrzna",
                "charakter_formalny__nazwa",
                "typ_kbn__nazwa",
                "zrodlo_lub_nadrzedne",
            )

        #
        ret = qset.only(*flds)

        sql = str(ret.query)

        if "bpp_autorzy_mat" in sql or "bpp_zewnetrzne_bazy_view" in sql:
            ret = ret.distinct()

        return ret

    def get_context_data(self, **kwargs):
        pass

        ctx = super().get_context_data()

        if not self.request.GET.get("print-removed", False):
            qset = self.get_queryset()
        else:
            qset = self.get_queryset(
                only_those_ids=self.request.session.get(
                    MULTISEEK_SESSION_KEY_REMOVED, []
                )
            )
            ctx["object_list"] = qset
            ctx["print_removed"] = True

        ctx["paginator_count"] = qset.count()
        object_list = ctx["object_list"]
        object_list.count = lambda *args, **kw: ctx["paginator_count"]

        if ctx["report_type"] in EXTRA_TYPES:
            ctx["sumy"] = qset.aggregate(
                Sum("impact_factor"),
                Sum("liczba_cytowan"),
                Sum("punkty_kbn"),
                Sum("index_copernicus"),
                Sum("punktacja_wewnetrzna"),
            )

        keys = list(self.request.session.keys())
        if "MULTISEEK_TITLE" not in keys:
            self.request.session["MULTISEEK_TITLE"] = "Rezultat wyszukiwania"
        else:
            if self.request.session["MULTISEEK_TITLE"] == "":
                self.request.session["MULTISEEK_TITLE"] = "Rezultat wyszukiwania"

        return ctx


@never_cache
def bpp_remove_by_hand(request, pk):
    """Add a record's PK to a list of manually removed records.

    User, via the web ui, can add or remove a record to a list of records
    removed "by hand". Those records will be explictly removed
    from the search results in the query function. The list of those
    records is cleaned when there is a form reset.
    """
    pk = tuple(int(x) for x in pk.split("_"))
    return manually_add_or_remove(request, pk)


@never_cache
def bpp_remove_from_removed_by_hand(request, pk):
    """Cancel manual record removal."""
    pk = tuple(int(x) for x in pk.split("_"))
    return manually_add_or_remove(request, pk, add=False)



import pytest
from selenium.webdriver.support.wait import WebDriverWait

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from model_bakery import baker
from selenium.webdriver.common.keys import Keys

from bpp.models import Autor, Zrodlo

from django_bpp.selenium_util import SHORT_WAIT_TIME, wait_for_page_load

pytestmark = [pytest.mark.slow, pytest.mark.selenium]


@pytest.fixture(scope="function")
def autorzy_browser(browser, live_server):
    baker.make(Autor, nazwisko="Atest", imiona="foo")
    browser.visit(live_server + reverse("bpp:browse_autorzy"))
    yield browser
    browser.quit()


def test_autorzy_index(autorzy_browser):
    assert "Atest" in autorzy_browser.html


def test_autorzy_search_form(autorzy_browser):
    autorzy_browser.fill("search", "Atest")
    autorzy_browser.find_by_name("search").type(Keys.RETURN)
    autorzy_browser.wait_for_condition(lambda browser: "Atest" in browser.html)

    autorzy_browser.reload()
    autorzy_browser.fill("search", "Btest")
    autorzy_browser.find_by_name("search").type(Keys.RETURN)
    autorzy_browser.wait_for_condition(lambda browser: "Atest" not in browser.html)


def test_autorzy_literki(autorzy_browser):
    literka = autorzy_browser.find_by_id("literka_A")

    with wait_for_page_load(autorzy_browser):
        literka.click()
    assert "Atest" in autorzy_browser.html

    literka = autorzy_browser.find_by_id("literka_B")
    with wait_for_page_load(autorzy_browser):
        literka.click()
    assert "Atest" not in autorzy_browser.html


@pytest.fixture(scope="function")
def zrodla_browser(browser, live_server):
    baker.make(Zrodlo, nazwa="Atest")
    browser.visit(live_server + reverse("bpp:browse_zrodla"))
    yield browser
    browser.quit()


def test_zrodla_index(zrodla_browser):
    assert "Atest" in zrodla_browser.html


def test_zrodla_literki(zrodla_browser):
    literka = zrodla_browser.find_by_id("literka_A")

    with wait_for_page_load(zrodla_browser):
        literka.click()
    assert "Atest" in zrodla_browser.html

    literka = zrodla_browser.find_by_id("literka_B")
    with wait_for_page_load(zrodla_browser):
        literka.click()
    assert "Atest" not in zrodla_browser.html


def test_zrodla_search_form(zrodla_browser):
    input = zrodla_browser.find_by_name("search")
    input.type("Atest")
    input.type(Keys.RETURN)
    WebDriverWait(zrodla_browser.driver, SHORT_WAIT_TIME).until(
        lambda x: "Atest" in zrodla_browser.html
    )

    zrodla_browser.reload()
    input = zrodla_browser.find_by_name("search")
    input.fill("Btest")
    input.type(Keys.RETURN)

    WebDriverWait(zrodla_browser.driver, SHORT_WAIT_TIME).until(
        lambda x: "Atest" not in zrodla_browser.html
    )



# Generated by Django 3.0.14 on 2021-10-10 21:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0297_liczba_n_uczelni"),
        ("bpp", "0304_auto_20211010_2247"),
    ]

    operations = []



# Generated by Django 3.0.9 on 2020-10-07 08:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0224_auto_20201007_0956"),
    ]

    operations = [
        migrations.AddField(
            model_name="ukryj_status_korekty",
            name="multiwyszukiwarka",
            field=models.BooleanField(
                default=True,
                help_text="Nie dotyczy użytkownika zalogowanego. Użytkownik zalogowany widzi wszystkie prace. ",
            ),
        ),
        migrations.AddField(
            model_name="ukryj_status_korekty",
            name="rankingi",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="ukryj_status_korekty",
            name="raporty",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="ukryj_status_korekty",
            name="sloty",
            field=models.BooleanField(default=True),
        ),
    ]



import pytest

from django.contrib.auth.models import Group

from bpp.const import GR_WPROWADZANIE_DANYCH, GR_ZGLOSZENIA_PUBLIKACJI
from bpp.core import editors_emails, zgloszenia_publikacji_emails
from bpp.models import BppUser


@pytest.mark.django_db
def test_editors_emails(admin_user: BppUser):
    ADMIN_EMAIL = "foo@bar.pl"

    g = Group.objects.get(name=GR_WPROWADZANIE_DANYCH)
    admin_user.email = ADMIN_EMAIL
    admin_user.save()

    admin_user.groups.add(g)

    assert ADMIN_EMAIL in editors_emails()


@pytest.mark.django_db
def test_zgloszenia_publikacji_emails(admin_user: BppUser, normal_django_user: BppUser):
    ADMIN_EMAIL = "foo@bar.pl"
    USER_EMAIL = "user@bar.pl"

    g = Group.objects.get(name=GR_WPROWADZANIE_DANYCH)
    admin_user.email = ADMIN_EMAIL
    admin_user.save()

    admin_user.groups.add(g)

    assert ADMIN_EMAIL in zgloszenia_publikacji_emails()

    g = Group.objects.get(name=GR_ZGLOSZENIA_PUBLIKACJI)
    normal_django_user.email = USER_EMAIL
    normal_django_user.save()

    normal_django_user.groups.add(g)
    assert ADMIN_EMAIL not in zgloszenia_publikacji_emails()
    assert USER_EMAIL in zgloszenia_publikacji_emails()



"""Funkcje API pomagające stronom synchronizacji danych z CrossRef API
"""

from braces.views import GroupRequiredMixin, JSONResponseMixin
from django.views.generic.edit import BaseFormView

from bpp.const import GR_WPROWADZANIE_DANYCH
from bpp.views.api.forms import (
    UstawNrZeszytuRekorduForm,
    UstawORCIDAutoraForm,
    UstawStreszczenieRekorduForm,
    UstawStronyRekorduForm,
    UstawTomRekorduForm,
)


class UstawParametrBaseView(JSONResponseMixin, GroupRequiredMixin, BaseFormView):
    group_required = GR_WPROWADZANIE_DANYCH

    def form_invalid(self, form):
        return self.render_json_response({"status": "error", "errors": form.errors})

    def form_valid(self, form):
        self.modify_object(form.cleaned_data)
        return self.render_json_response({"status": "ok"})

    def modify_object(self, data):
        raise NotImplementedError


class UstawiORCIDAutoraView(UstawParametrBaseView):
    form_class = UstawORCIDAutoraForm

    def modify_object(self, data):
        autor = data["autor"]
        autor.orcid = data["orcid"]
        autor.save()


class UstawStronyView(UstawParametrBaseView):
    form_class = UstawStronyRekorduForm

    def modify_object(self, data):
        rekord = data["rekord"]

        ro = rekord.original
        ro.strony = data["strony"]
        ro.save()


class UstawTomView(UstawParametrBaseView):
    form_class = UstawTomRekorduForm

    def modify_object(self, data):
        rekord = data["rekord"]

        ro = rekord.original
        ro.tom = data["tom"]
        ro.save()


class UstawNrZeszytuView(UstawParametrBaseView):
    form_class = UstawNrZeszytuRekorduForm

    def modify_object(self, data):
        rekord = data["rekord"]

        ro = rekord.original
        ro.nr_zeszytu = data["nr_zeszytu"]
        ro.save()


class UstawStreszczenieView(UstawParametrBaseView):
    form_class = UstawStreszczenieRekorduForm

    def modify_object(self, data):
        rekord = data["rekord"]

        ro = rekord.original
        if not ro.streszczenia.filter(streszczenie=data["streszczenie"]).exists():
            ro.streszczenia.create(streszczenie=data["streszczenie"])



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-11-10 23:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0114_auto_20171111_0036'),
    ]

    operations = [
        migrations.AlterField(
            model_name='autor',
            name='pokazuj',
            field=models.BooleanField(default=True, help_text='Pokazuj autora na stronach jednostek oraz w rankingu. '),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-12 22:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0136_ranking_liczba_cytowan'),
    ]

    operations = [
        migrations.AlterField(
            model_name='praca_doktorska',
            name='liczba_cytowan',
            field=models.PositiveIntegerField(blank=True, help_text="Wartość aktualizowana jest automatycznie raz na kilka dni w przypadku \n        skonfigurowania dostępu do API WOS AMR (przez obiekt 'Uczelnia'). Możesz również\n        czaktualizować tą wartość ręcznie, naciskając przycisk. ", null=True, verbose_name='Liczba cytowań'),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='liczba_cytowan',
            field=models.PositiveIntegerField(blank=True, help_text="Wartość aktualizowana jest automatycznie raz na kilka dni w przypadku \n        skonfigurowania dostępu do API WOS AMR (przez obiekt 'Uczelnia'). Możesz również\n        czaktualizować tą wartość ręcznie, naciskając przycisk. ", null=True, verbose_name='Liczba cytowań'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='liczba_cytowan',
            field=models.PositiveIntegerField(blank=True, help_text="Wartość aktualizowana jest automatycznie raz na kilka dni w przypadku \n        skonfigurowania dostępu do API WOS AMR (przez obiekt 'Uczelnia'). Możesz również\n        czaktualizować tą wartość ręcznie, naciskając przycisk. ", null=True, verbose_name='Liczba cytowań'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='liczba_cytowan',
            field=models.PositiveIntegerField(blank=True, help_text="Wartość aktualizowana jest automatycznie raz na kilka dni w przypadku \n        skonfigurowania dostępu do API WOS AMR (przez obiekt 'Uczelnia'). Możesz również\n        czaktualizować tą wartość ręcznie, naciskając przycisk. ", null=True, verbose_name='Liczba cytowań'),
        ),
    ]



from django.urls import reverse

from bpp.models import Uczelnia


def test_admin_struktura_JednostkaAdmin_uczelnia_jednostki_alfabetycznie(
    uczelnia: Uczelnia, admin_client
):
    uczelnia.sortuj_jednostki_alfabetycznie = True
    uczelnia.save()

    assert (
        admin_client.get(reverse("admin:bpp_jednostka_changelist")).status_code == 200
    )


def test_admin_struktura_JednostkaAdmin_uczelnia_jednostki_wg_kolejnosci(
    uczelnia: Uczelnia, admin_client
):
    uczelnia.sortuj_jednostki_alfabetycznie = False
    uczelnia.save()

    assert (
        admin_client.get(reverse("admin:bpp_jednostka_changelist")).status_code == 200
    )



# -*- encoding: utf-8 -*-
import os
import sys
import shutil

from django.core.management import BaseCommand
from django.db import transaction

from bpp.reports.opi_2012 import make_report_zipfile


class Command(BaseCommand):
    help = 'Eksportuje raporty OPI 2009-2012'

    @transaction.atomic
    def handle(self, *args, **options):
        wydzialy = ['1WL', '2WL', 'WF', 'WP']
        zipname = make_report_zipfile(wydzialy=wydzialy)

        if sys.platform == 'win32':
            shutil.move(
                zipname,
                os.path.join(os.getenv("USERPROFILE"), 'Desktop'))




# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0059_auto_20161114_0945'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wydzial',
            name='otwarcie',
            field=models.DateField(null=True, verbose_name=b'Data otwarcia wydzia\xc5\x82u', blank=True),
        ),
    ]



from django.db import models


class BppMultiseekVisibility(models.Model):
    field_name = models.CharField(
        "Systemowa nazwa pola", max_length=50, db_index=True, unique=True
    )
    label = models.CharField("Nazwa pola", max_length=200)
    public = models.BooleanField(
        default=True, verbose_name="Widoczne dla niezalogowanych"
    )

    authenticated = models.BooleanField(
        default=True, verbose_name="Widoczne dla zalogowanych"
    )
    staff = models.BooleanField(
        default=True, verbose_name='Widoczne dla osób "w zespole"'
    )

    sort_order = models.PositiveSmallIntegerField("Kolejność sortowania", default=0)

    class Meta:
        verbose_name = "widoczność opcji wyszukiwania"
        verbose_name_plural = "widoczność opcji wyszukiwania"
        ordering = ("sort_order",)

    def __str__(self):
        return f'Widoczność opcji wyszukiwania dla "{self.label}"'



from rest_framework import serializers
from taggit_serializer.serializers import TagListSerializerField

from api_v1.serializers.util import (
    AbsoluteUrlSerializerMixin,
    Wydawnictwo_AutorSerializerMixin,
    WydawnictwoSerializerMixin,
)

from bpp.models import (
    Wydawnictwo_Zwarte,
    Wydawnictwo_Zwarte_Autor,
    Wydawnictwo_Zwarte_Streszczenie,
)


class Wydawnictwo_Zwarte_AutorSerializer(
    Wydawnictwo_AutorSerializerMixin, serializers.HyperlinkedModelSerializer
):
    rekord = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydawnictwo_zwarte-detail", read_only=True
    )

    class Meta:
        model = Wydawnictwo_Zwarte_Autor
        fields = [
            "id",
            "autor",
            "jednostka",
            "zapisany_jako",
            "typ_odpowiedzialnosci",
            "afiliuje",
            "zatrudniony",
            "kolejnosc",
            "rekord",
            "procent",
            "dyscyplina_naukowa",
        ]


class Wydawnictwo_Zwarte_StreszczenieSerializer(serializers.ModelSerializer):
    rekord = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydawnictwo_zwarte-detail", read_only=True
    )

    class Meta:
        model = Wydawnictwo_Zwarte_Streszczenie
        fields = ["rekord", "jezyk_streszczenia", "streszczenie"]


class Wydawnictwo_ZwarteSerializer(
    AbsoluteUrlSerializerMixin,
    WydawnictwoSerializerMixin,
    serializers.HyperlinkedModelSerializer,
):
    slowa_kluczowe = TagListSerializerField()

    wydawca = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydawca-detail", read_only=True
    )

    autorzy_set = serializers.HyperlinkedRelatedField(
        many=True, view_name="api_v1:wydawnictwo_zwarte_autor-detail", read_only=True
    )

    konferencja = serializers.HyperlinkedRelatedField(
        view_name="api_v1:konferencja-detail", read_only=True
    )

    seria_wydawnicza = serializers.HyperlinkedRelatedField(
        view_name="api_v1:seria_wydawnicza-detail", read_only=True
    )

    wydawnictwo_nadrzedne = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydawnictwo_zwarte-detail", read_only=True
    )

    streszczenia = serializers.HyperlinkedRelatedField(
        many=True,
        view_name="api_v1:wydawnictwo_zwarte_streszczenie-detail",
        read_only=True,
    )

    class Meta:
        model = Wydawnictwo_Zwarte
        fields = [
            "id",
            #
            "tytul_oryginalny",
            "tytul",
            #
            "rok",
            "status_korekty",
            #
            "wydawnictwo_nadrzedne",
            #
            "jezyk",
            "jezyk_alt",
            "charakter_formalny",
            "typ_kbn",
            #
            "www",
            "dostep_dnia",
            "public_www",
            "public_dostep_dnia",
            #
            "pubmed_id",
            "pmc_id",
            "doi",
            "pbn_id",
            #
            "praca_wybitna",
            "uzasadnienie_wybitnosci",
            #
            "konferencja",
            #
            "seria_wydawnicza",
            "numer_w_serii",
            #
            "impact_factor",
            "punkty_kbn",
            #
            "informacje",
            "szczegoly",
            "uwagi",
            "slowa_kluczowe",
            "strony",
            "tom",
            #
            "isbn",
            "e_isbn",
            #
            "liczba_cytowan",
            #
            "oznaczenie_wydania",
            "miejsce_i_rok",
            "wydawca",
            "wydawca_opis",
            "redakcja",
            #
            "openaccess_tryb_dostepu",
            "openaccess_wersja_tekstu",
            "openaccess_licencja",
            "openaccess_czas_publikacji",
            "openaccess_ilosc_miesiecy",
            #
            "utworzono",
            "ostatnio_zmieniony",
            #
            "absolute_url",
            #
            "tekst_przed_pierwszym_autorem",
            "autorzy_set",
            "tekst_po_ostatnim_autorze",
            #
            "nagrody",
            "streszczenia",
        ]



import pytest
from model_bakery import baker

from fixtures.pbn_api import MOCK_RETURNED_MONGODB_DATA
from pbn_api.adapters.wydawnictwo import WydawnictwoPBNAdapter
from pbn_api.client import (
    PBN_DELETE_PUBLICATION_STATEMENT,
    PBN_GET_INSTITUTION_STATEMENTS,
    PBN_GET_PUBLICATION_BY_ID_URL,
    PBN_POST_PUBLICATIONS_URL,
)
from pbn_api.exceptions import (
    HttpException,
    PKZeroExportDisabled,
    SameDataUploadedRecently,
)
from pbn_api.models import Institution, Publication, SentData
from pbn_api.tests.utils import middleware

from django.contrib.messages import get_messages

from bpp.admin.helpers import sprobuj_wgrac_do_pbn
from bpp.decorators import json


@pytest.mark.django_db
def test_PBNClient_test_upload_publication_nie_trzeba(
    pbn_client, pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina
):
    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {"objectId": None}

    SentData.objects.updated(
        pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina,
        WydawnictwoPBNAdapter(
            pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina
        ).pbn_get_json(),
    )

    with pytest.raises(SameDataUploadedRecently):
        pbn_client.upload_publication(pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina)


class PBNTestClientException(Exception):
    pass


@pytest.mark.django_db
def test_PBNClient_test_upload_publication_exception(
    pbn_client, pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina
):
    pbn_client.transport.return_values[
        PBN_POST_PUBLICATIONS_URL
    ] = PBNTestClientException("nei")

    with pytest.raises(PBNTestClientException):
        pbn_client.upload_publication(pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina)


@pytest.mark.django_db
def test_PBNClient_test_upload_publication_wszystko_ok(
    pbn_client, pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina, pbn_publication
):

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {
        "objectId": pbn_publication.pk
    }

    ret, js = pbn_client.upload_publication(
        pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina
    )
    assert ret["objectId"] == pbn_publication.pk


@pytest.mark.django_db
def test_sync_publication_to_samo_id(
    pbn_client,
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina,
    pbn_publication,
    pbn_autor,
    pbn_jednostka,
):
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid = pbn_publication
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.save()

    stare_id = pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {
        "objectId": pbn_publication.pk
    }
    pbn_client.transport.return_values[
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=pbn_publication.pk)
    ] = MOCK_RETURNED_MONGODB_DATA
    pbn_client.transport.return_values[
        PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=123&size=5120"
    ] = [
        {
            "id": "100",
            "addedTimestamp": "2020.05.06",
            "institutionId": pbn_jednostka.pbn_uid_id,
            "personId": pbn_autor.pbn_uid_id,
            "publicationId": pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id,
            "area": "200",
            "inOrcid": True,
            "type": "FOOBAR",
        }
    ]

    pbn_client.sync_publication(pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina)

    pbn_publication.refresh_from_db()
    assert pbn_publication.versions[0]["baz"] == "quux"
    assert stare_id == pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id


@pytest.mark.django_db
def test_sync_publication_tekstowo_podane_id(
    pbn_client, pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina, pbn_publication
):

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {
        "objectId": pbn_publication.pk
    }
    pbn_client.transport.return_values[
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=pbn_publication.pk)
    ] = MOCK_RETURNED_MONGODB_DATA
    pbn_client.transport.return_values[
        PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=123&size=5120"
    ] = []

    pbn_client.sync_publication(
        f"wydawnictwo_zwarte:{pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pk}"
    )

    pbn_publication.refresh_from_db()
    assert pbn_publication.versions[0]["baz"] == "quux"


@pytest.mark.django_db
def test_sync_publication_nowe_id(
    pbn_client, pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina, pbn_publication
):
    assert pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id is None

    stare_id = pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {
        "objectId": pbn_publication.pk
    }
    pbn_client.transport.return_values[
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=pbn_publication.pk)
    ] = MOCK_RETURNED_MONGODB_DATA
    pbn_client.transport.return_values[
        PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=123&size=5120"
    ] = []

    pbn_client.sync_publication(pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina)

    pbn_publication.refresh_from_db()
    assert pbn_publication.versions[0]["baz"] == "quux"
    assert stare_id != pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id


@pytest.mark.django_db
def test_sync_publication_wysylka_z_zerowym_pk(
    pbn_client,
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina,
    pbn_publication,
    pbn_uczelnia,
):
    pbn_uczelnia.pbn_api_nie_wysylaj_prac_bez_pk = True
    pbn_uczelnia.save()

    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.punkty_kbn = 0
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.save()

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {
        "objectId": pbn_publication.pk
    }
    pbn_client.transport.return_values[
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=pbn_publication.pk)
    ] = MOCK_RETURNED_MONGODB_DATA
    pbn_client.transport.return_values[
        PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=123&size=5120"
    ] = []

    # To pójdzie
    pbn_client.sync_publication(
        pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina, export_pk_zero=True
    )

    # To nie pójdzie
    with pytest.raises(PKZeroExportDisabled):
        pbn_client.sync_publication(
            pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina, export_pk_zero=False
        )


@pytest.mark.django_db
def test_helpers_wysylka_z_zerowym_pk(
    rf, pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina, pbn_uczelnia, admin_user
):
    pbn_uczelnia.pbn_integracja = (
        pbn_uczelnia.pbn_aktualizuj_na_biezaco
    ) = pbn_uczelnia.pbn_api_nie_wysylaj_prac_bez_pk = True
    pbn_uczelnia.save()

    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.punkty_kbn = 0
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.save()

    req = rf.get("/")
    req._uczelnia = pbn_uczelnia
    req.user = admin_user

    # I jeszcze test z poziomu admina czy parametr z pbn_uczelnia jest przekazywany
    with middleware(req):
        sprobuj_wgrac_do_pbn(req, pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina)
        msg = list(get_messages(req))

    assert "wyłączony w konfiguracji" in msg[0].message


@pytest.mark.django_db
def test_helpers_wysylka_z_uid_uczelni(
    rf,
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina,
    pbn_autor_z_dyscyplina,
    pbn_uczelnia,
    admin_user,
    pbn_client,
):
    odpowiednik = baker.make(Institution, mongoId="PBN_UID_UCZELNI----")

    baker.make(Publication, mongoId=pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pk)

    pbn_uczelnia.pbn_uid = odpowiednik
    pbn_uczelnia.pbn_api_afiliacja_zawsze_na_uczelnie = True
    pbn_uczelnia.save()

    pbn_uczelnia.pbn_integracja = pbn_uczelnia.pbn_aktualizuj_na_biezaco = True
    pbn_uczelnia.save()

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {
        "objectId": pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pk
    }
    pbn_client.transport.return_values[
        PBN_GET_PUBLICATION_BY_ID_URL.format(
            id=pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pk
        )
    ] = MOCK_RETURNED_MONGODB_DATA

    pbn_client.transport.return_values[
        PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=123&size=5120"
    ] = []

    req = rf.get("/")
    req._uczelnia = pbn_uczelnia
    req.user = admin_user

    with middleware(req):
        sprobuj_wgrac_do_pbn(req, pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina)
        msg = list(get_messages(req))

    assert len(msg) == 1 and str(msg[0]).find("y zaktualizowane") > -1

    iv = pbn_client.transport.input_values["/api/v1/publications"]
    assert iv["body"]["authors"][0]["affiliations"][0] == odpowiednik.pk
    assert iv["body"]["institutions"][odpowiednik.pk]["objectId"] == odpowiednik.pk


@pytest.mark.django_db
def test_helpers_wysylka_bez_uid_uczelni(
    rf,
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina,
    pbn_autor_z_dyscyplina,
    pbn_uczelnia,
    pbn_jednostka,
    admin_user,
    pbn_client,
):
    odpowiednik = baker.make(Institution, mongoId="PBN_UID_UCZELNI----")

    baker.make(Publication, mongoId=pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pk)

    pbn_uczelnia.pbn_uid = odpowiednik
    pbn_uczelnia.pbn_api_afiliacja_zawsze_na_uczelnie = False
    pbn_uczelnia.save()

    pbn_uczelnia.pbn_integracja = pbn_uczelnia.pbn_aktualizuj_na_biezaco = True
    pbn_uczelnia.save()

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {
        "objectId": pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pk
    }
    pbn_client.transport.return_values[
        PBN_GET_PUBLICATION_BY_ID_URL.format(
            id=pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pk
        )
    ] = MOCK_RETURNED_MONGODB_DATA

    pbn_client.transport.return_values[
        PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=123&size=5120"
    ] = []

    req = rf.get("/")
    req._uczelnia = pbn_uczelnia
    req.user = admin_user

    with middleware(req):
        sprobuj_wgrac_do_pbn(req, pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina)
        msg = list(get_messages(req))

    assert len(msg) == 1 and str(msg[0]).find("y zaktualizowane") > -1

    iv = pbn_client.transport.input_values["/api/v1/publications"]
    assert iv["body"]["authors"][0]["affiliations"][0] == pbn_jednostka.pbn_uid_id
    assert (
        iv["body"]["institutions"][pbn_jednostka.pbn_uid_id]["objectId"]
        == pbn_jednostka.pbn_uid_id
    )


@pytest.mark.django_db
def test_sync_publication_kasuj_oswiadczenia_przed_wszystko_dobrze(
    pbn_client,
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina,
    pbn_publication,
    pbn_autor,
    pbn_jednostka,
):
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid = pbn_publication
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.save()

    stare_id = pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {
        "objectId": pbn_publication.pk
    }
    pbn_client.transport.return_values[
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=pbn_publication.pk)
    ] = MOCK_RETURNED_MONGODB_DATA
    pbn_client.transport.return_values[
        PBN_DELETE_PUBLICATION_STATEMENT.format(publicationId=pbn_publication.pk)
    ] = []
    pbn_client.transport.return_values[
        PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=123&size=5120"
    ] = [
        {
            "id": "100",
            "addedTimestamp": "2020.05.06",
            "institutionId": pbn_jednostka.pbn_uid_id,
            "personId": pbn_autor.pbn_uid_id,
            "publicationId": pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id,
            "area": "200",
            "inOrcid": True,
            "type": "FOOBAR",
        }
    ]

    pbn_client.sync_publication(
        pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina,
        delete_statements_before_upload=True,
    )

    pbn_publication.refresh_from_db()
    assert pbn_publication.versions[0]["baz"] == "quux"
    assert stare_id == pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id


@pytest.mark.django_db
def test_sync_publication_kasuj_oswiadczenia_przed_blad_400_nie_zaburzy(
    pbn_client,
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina,
    pbn_publication,
    pbn_autor,
    pbn_jednostka,
):
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid = pbn_publication
    pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.save()

    stare_id = pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {
        "objectId": pbn_publication.pk
    }
    pbn_client.transport.return_values[
        PBN_GET_PUBLICATION_BY_ID_URL.format(id=pbn_publication.pk)
    ] = MOCK_RETURNED_MONGODB_DATA

    url = PBN_DELETE_PUBLICATION_STATEMENT.format(publicationId=pbn_publication.pk)
    err_json = {
        "code": 400,
        "message": "Bad Request",
        "description": "Validation failed.",
        "details": {
            "publicationId": "Nie można usunąć oświadczeń. Nie istnieją oświadczenia dla publikacji "
            "(id = {pbn_publication.pk}) i instytucji (id = XXX)."
        },
    }

    pbn_client.transport.return_values[url] = HttpException(
        400, url, json.dumps(err_json)
    )

    pbn_client.transport.return_values[
        PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=123&size=5120"
    ] = [
        {
            "id": "100",
            "addedTimestamp": "2020.05.06",
            "institutionId": pbn_jednostka.pbn_uid_id,
            "personId": pbn_autor.pbn_uid_id,
            "publicationId": pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id,
            "area": "200",
            "inOrcid": True,
            "type": "FOOBAR",
        }
    ]

    pbn_client.sync_publication(
        pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina,
        delete_statements_before_upload=True,
    )

    pbn_publication.refresh_from_db()
    assert pbn_publication.versions[0]["baz"] == "quux"
    assert stare_id == pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina.pbn_uid_id



from django.urls import reverse

from bpp.models import Autor_Dyscyplina, Autor_Jednostka
from bpp.tests import proper_click_by_id, select_select2_autocomplete

from django_bpp.selenium_util import wait_for_page_load


def test_zglos_publikacje_drugi_autor_dyscyplina(
    admin_browser,
    live_server,
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    dyscyplina1,
    rok,
):
    for autor in autor_jan_kowalski, autor_jan_nowak:
        Autor_Dyscyplina.objects.get_or_create(
            autor=autor, rok=rok, dyscyplina_naukowa=dyscyplina1
        )
        Autor_Jednostka.objects.get_or_create(autor=autor, jednostka=jednostka)

    admin_browser.visit(live_server.url + reverse("zglos_publikacje:nowe_zgloszenie"))

    admin_browser.fill("0-tytul_oryginalny", "test")
    admin_browser.select("0-rodzaj_zglaszanej_publikacji", "2")
    admin_browser.fill("0-strona_www", "https://www.onet.pl/")
    admin_browser.fill("0-rok", str(rok))
    admin_browser.fill("0-email", "moj@email.pl")
    with wait_for_page_load(admin_browser):
        proper_click_by_id(admin_browser, "id-wizard-submit")

    n = 1
    admin_browser.find_by_id("add-form").click()
    select_select2_autocomplete(admin_browser, f"id_2-{n}-autor", "Kowal")
    assert admin_browser.find_by_id(f"id_2-{n}-dyscyplina_naukowa").value == str(
        dyscyplina1.pk
    )


def test_zglos_publikacje_z_plikiem_drugi_autor_dyscyplina(
    admin_browser,
    live_server,
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    dyscyplina1,
    rok,
):
    for autor in autor_jan_kowalski, autor_jan_nowak:
        Autor_Dyscyplina.objects.get_or_create(
            autor=autor, rok=rok, dyscyplina_naukowa=dyscyplina1
        )
        Autor_Jednostka.objects.get_or_create(autor=autor, jednostka=jednostka)

    admin_browser.visit(live_server.url + reverse("zglos_publikacje:nowe_zgloszenie"))

    admin_browser.fill("0-tytul_oryginalny", "test")
    admin_browser.select("0-rodzaj_zglaszanej_publikacji", "2")
    admin_browser.fill("0-rok", str(rok))
    admin_browser.fill("0-email", "moj@email.pl")
    with wait_for_page_load(admin_browser):
        proper_click_by_id(admin_browser, "id-wizard-submit")

    import os

    plik = os.path.abspath(os.path.join(os.path.dirname(__file__), "example.pdf"))
    admin_browser.fill("1-plik", plik)
    with wait_for_page_load(admin_browser):
        proper_click_by_id(admin_browser, "id-wizard-submit")

    n = 1
    admin_browser.find_by_id("add-form").click()
    select_select2_autocomplete(admin_browser, f"id_2-{n}-autor", "Kowal")
    assert admin_browser.find_by_id(f"id_2-{n}-dyscyplina_naukowa").value == str(
        dyscyplina1.pk
    )


def test_zglos_publikacje_wiele_klikniec_psuje_select2(
    admin_browser,
    live_server,
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    dyscyplina1,
    rok,
):
    for autor in autor_jan_kowalski, autor_jan_nowak:
        Autor_Dyscyplina.objects.get_or_create(
            autor=autor, rok=rok, dyscyplina_naukowa=dyscyplina1
        )
        Autor_Jednostka.objects.get_or_create(autor=autor, jednostka=jednostka)

    admin_browser.visit(live_server.url + reverse("zglos_publikacje:nowe_zgloszenie"))

    admin_browser.fill("0-tytul_oryginalny", "test")
    admin_browser.select("0-rodzaj_zglaszanej_publikacji", "2")
    admin_browser.fill("0-strona_www", "https://www.onet.pl/")
    admin_browser.fill("0-rok", str(rok))
    admin_browser.fill("0-email", "moj@email.pl")
    with wait_for_page_load(admin_browser):
        proper_click_by_id(admin_browser, "id-wizard-submit")

    proper_click_by_id(admin_browser, "add-form")
    proper_click_by_id(admin_browser, "add-form")
    proper_click_by_id(admin_browser, "add-form")

    select_select2_autocomplete(admin_browser, "id_2-1-autor", "Kowal")
    assert admin_browser.find_by_id("id_2-1-dyscyplina_naukowa").value == str(
        dyscyplina1.pk
    )


def test_zglos_publikacje_z_plikiem_wiele_klikniec_psuje_select2(
    admin_browser,
    live_server,
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    dyscyplina1,
    rok,
):
    for autor in autor_jan_kowalski, autor_jan_nowak:
        Autor_Dyscyplina.objects.get_or_create(
            autor=autor, rok=rok, dyscyplina_naukowa=dyscyplina1
        )
        Autor_Jednostka.objects.get_or_create(autor=autor, jednostka=jednostka)

    admin_browser.visit(live_server.url + reverse("zglos_publikacje:nowe_zgloszenie"))

    admin_browser.fill("0-tytul_oryginalny", "test")
    admin_browser.select("0-rodzaj_zglaszanej_publikacji", "2")
    admin_browser.fill("0-rok", str(rok))
    admin_browser.fill("0-email", "moj@email.pl")
    with wait_for_page_load(admin_browser):
        proper_click_by_id(admin_browser, "id-wizard-submit")

    import os

    plik = os.path.abspath(os.path.join(os.path.dirname(__file__), "example.pdf"))
    admin_browser.fill("1-plik", plik)
    with wait_for_page_load(admin_browser):
        proper_click_by_id(admin_browser, "id-wizard-submit")

    proper_click_by_id(admin_browser, "add-form")
    proper_click_by_id(admin_browser, "add-form")
    proper_click_by_id(admin_browser, "add-form")

    select_select2_autocomplete(admin_browser, "id_2-1-autor", "Kowal")
    assert admin_browser.find_by_id("id_2-1-dyscyplina_naukowa").value == str(
        dyscyplina1.pk
    )



from collections import defaultdict
from typing import Generator, List

import openpyxl
from openpyxl.utils.exceptions import InvalidFileException
from openpyxl.worksheet.worksheet import Worksheet

from .exceptions import (
    BadNoOfSheetsException,
    HeaderNotFoundException,
    ImproperFileException,
)

from django.utils.functional import cached_property

DEFAULT_COL_NAMES = [
    "imię",
    "imie",
    "imiona",
    "nazwisko",
    "nazwiska",
    "orcid",
    "pesel",
    "pbn_id",
    "stanowisko",
    "wydział",
    "jednostka",
    "numer",
]

DEFAULT_MIN_POINTS = 3


def normalize_cell_header(elem: openpyxl.cell.cell.Cell):
    s = str(elem.value).lower().split("\n")[0]

    s = s.replace(".", " ")
    while s.find("  ") >= 0:
        s = s.replace("  ", " ")
    s = s.strip()

    return s.replace(" ", "_").replace("/", "_").replace("\\", "_").replace("-", "_")


def find_similar_row(
    sheet: Worksheet, try_names=None, min_points=None, max_row_length=128
):
    if try_names is None:
        try_names = DEFAULT_COL_NAMES

    if min_points is None:
        min_points = DEFAULT_MIN_POINTS

    for n, row in enumerate(sheet.rows, start=1):
        r = [normalize_cell_header(elem) for elem in row[:max_row_length]]
        points = 0
        for elem in try_names:
            if elem in r:
                points += 1
        if points >= min_points:
            return r, n


def znajdz_naglowek(
    sciezka,
    try_names=None,
    min_points=None,
):
    """
    :return: ([str, str...], no_row)
    """
    try:
        f: openpyxl.workbook.workbook.Workbook = openpyxl.load_workbook(sciezka)
    except InvalidFileException as e:
        raise ImproperFileException(e)

    # Sprawdź, ile jest skoroszytów
    if len(f.worksheets) != 1:
        raise BadNoOfSheetsException()

    s = f.worksheets[0]

    res = find_similar_row(s, try_names, min_points)

    if res is None:
        raise HeaderNotFoundException()

    return res


DEFAULT_BANNED_NAMES = ["pesel", "pesel_md5", "peselmd5"]


class XLSImportFile:
    def __init__(
        self,
        xls_path,
        try_names=None,
        min_points=None,
        banned_names=None,
        only_first_sheet=False,
    ):
        """
        :param xls_path: ścieżka do pliku
        :param try_names: nazwy które będą poszukiwane jako nagłówek
        :param min_points: ile z nazw nagłówka musi się odnaleźć w wierszu, żeby był tak potraktowany
        :param banned_names: niedozwolone nazwy nagłówków - te kolumny nie będą importowane (np PESEL)
        """

        self.xls_path = xls_path
        self.try_names = try_names
        self.min_points = min_points

        if banned_names is None:
            banned_names = DEFAULT_BANNED_NAMES
        self.banned_names = banned_names

        self.only_first_sheet = only_first_sheet

    @cached_property
    def xl_workbook(self) -> openpyxl.workbook.workbook.Workbook:
        return openpyxl.load_workbook(self.xls_path)

    @cached_property
    def sheet_limit_range_end(self):
        limit = None
        if self.only_first_sheet:
            limit = 1
        return limit

    @cached_property
    def sheet_row_cache(self):
        _cache = {}

        for n_sheet, sheet in enumerate(
            self.xl_workbook.worksheets[: self.sheet_limit_range_end]
        ):
            res = find_similar_row(
                sheet, try_names=self.try_names, min_points=self.min_points
            )
            if res is None:
                continue
            _cache[sheet] = res
        return _cache

    def count(self) -> int:
        """
        Zwraca całkowitą liczbę wierszy do analizy
        """
        total = 0
        for n_sheet, sheet in enumerate(self.xl_workbook.worksheets):
            res = self.sheet_row_cache.get(sheet)
            if res is None:
                continue
            labels, no = res
            total += sheet.max_row - no

        return total

    def data(self) -> Generator[dict, None, None]:
        """
        Ta funkcja dla każdego arkusza:
        1) znajduje nagłówek za pomoca funkcji `find_similar_row`,
        2) wczytuje poniższe wiersze do końca arkusza,
        3) dla każdego wiersza przypisuje nazwy kolumn z wiersza nagłówkowego
        4) zwraca słowniki.
        """

        for n_sheet, sheet in enumerate(self.xl_workbook.worksheets):

            res = self.sheet_row_cache.get(sheet)
            if res is None:
                continue

            colnames, no = res

            colnames = rename_duplicate_columns(colnames)

            colnames.append("__xls_loc_sheet__")
            colnames.append("__xls_loc_row__")

            for n_row, row in enumerate(sheet.rows):
                if n_row < res[1]:
                    continue
                data = [x.value for x in row[: len(colnames) - 2]]
                data.append(n_sheet)
                data.append(n_row)

                yld = dict(zip(colnames, data))

                for banned_name in self.banned_names:
                    if banned_name in yld:
                        del yld[banned_name]

                yield yld


def rename_duplicate_columns(s: List[str], marker: str = "_") -> List[str]:
    seen = defaultdict(lambda: 1)
    ret = []
    for elem in s:
        no_seen = seen[elem]
        if no_seen == 1:
            ret.append(elem)
        else:
            ret.append(f"{elem}{marker}{no_seen}")
        seen[elem] += 1
    return ret



# Generated by Django 3.0.14 on 2021-08-08 21:34

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0021_auto_20210808_1204"),
        ("bpp", "0281_auto_20210725_2332"),
    ]

    operations = [
        migrations.AddField(
            model_name="konferencja",
            name="pbn_uid",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="pbn_api.Conference",
                verbose_name="Odpowiednik w PBN",
            ),
        ),
    ]



# -*- encoding: utf-8 -*-
from datetime import date

import os

from django.core.management import BaseCommand
from django.db import transaction

from bpp.models import Autor_Jednostka

from bpp.imports.egeria_2012 import importuj_afiliacje
from bpp.imports.uml import UML_Egeria_2012_Mangle
from bpp.management.commands import files_or_directory

class Command(BaseCommand):
    help = 'Importuje afiliacje do wydziału z arkuszy XLS'
    args = '<katalog z xlsx> | <plik xls 1> <plik xls 2> ...'

    @transaction.atomic
    def handle(self, *args, **options):

        for plik_xls in files_or_directory(args):
            print((" *** Importuje plik", plik_xls))
            importuj_afiliacje(plik_xls, UML_Egeria_2012_Mangle)

        for aj in Autor_Jednostka.objects.filter(zakonczyl_prace=date(2012,12,31)):
            # Zakładamy, że ci ludzie pracują NADAL
            aj.zakonczyl_prace=None
            aj.save()



import openpyxl.worksheet.worksheet
from django.db.models import BooleanField, Case, F, Value, When

from ewaluacja2021.core.util import NieArtykul
from ewaluacja2021.util import output_table_to_xlsx

from bpp.models import Cache_Punktacja_Autora_Query
from bpp.util import strip_html


def get_data_for_report(qset):
    return (
        [
            # elem.id,
            str(elem.rekord_id),
            elem.autor_id,
            strip_html(elem.rekord.tytul_oryginalny),
            elem.rok,
            f"{elem.autor.nazwisko} {elem.autor.imiona}",
            elem.dyscyplina.nazwa,
            elem.monografia,
            elem.do_ewaluacji,
            elem.pkdaut,
            elem.slot,
            elem.rekord.punkty_kbn,
        ]
        for elem in qset
    )


def write_data_to_report(ws: openpyxl.worksheet.worksheet.Worksheet, data):
    output_table_to_xlsx(
        ws,
        "Przeszly",
        [
            # "ID elementu",
            "ID rekordu",
            "ID autora",
            "Tytuł",
            "Rok",
            "Autor",
            "Nazwa dyscypliny",
            "Monografia?",
            "Do ewaluacji",
            "PKDAut",
            "Slot",
            "PK",
        ],
        data,
        totals=["PKDAut", "Slot"],
        column_widths={
            "A": 10,
            "B": 14,
            "E": 14,
        },
    )


def rekordy(dane):
    return (
        Cache_Punktacja_Autora_Query.objects.filter(
            pk__in=[x["id"] for x in dane["wejscie"]]
        )
        .annotate(
            monografia=NieArtykul(
                F("rekord__charakter_formalny__rodzaj_pbn"), output_field=BooleanField()
            ),
            rok=F("rekord__rok"),
            do_ewaluacji=Case(
                When(pk__in=dane["optimum"], then=Value(True, BooleanField())),
                default=Value(False, BooleanField()),
            ),
        )
        .select_related("rekord", "autor", "dyscyplina")
    )


def load_data(fobj):
    import simplejson

    return simplejson.load(fobj, use_decimal=True)



from django.core.management import BaseCommand
from django.db import transaction

from ewaluacja2021.core.plecakowy import Plecakowy


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("--dyscyplina", default="nauki medyczne")
        parser.add_argument("--bez-limitu-uczelni", type=bool, default=False)
        parser.add_argument("--output-path", default=None, type=str)

    @transaction.atomic
    def handle(self, dyscyplina, output_path, bez_limitu_uczelni, *args, **options):

        algorytm = Plecakowy(
            nazwa_dyscypliny=dyscyplina,
            bez_limitu_uczelni=bez_limitu_uczelni,
            output_path=output_path,
        )
        algorytm.powitanie()
        algorytm.sumuj()
        algorytm.zrzuc_dane("plecakowy")
        algorytm.pozegnanie()



# Generated by Django 3.0.9 on 2020-10-07 07:56

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0223_auto_20200812_1930"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="element_repozytorium",
            options={
                "verbose_name": "element repozytorium",
                "verbose_name_plural": "elementy repozytorium",
            },
        ),
        migrations.CreateModel(
            name="Ukryj_Status_Korekty",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "status_korekty",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="bpp.Status_Korekty",
                    ),
                ),
                (
                    "uczelnia",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="bpp.Uczelnia"
                    ),
                ),
            ],
            options={"unique_together": {("uczelnia", "status_korekty")},},
        ),
    ]



import pytest

from bpp import const
from bpp.models import (
    Autor_Dyscyplina,
    Charakter_Formalny,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Zwarte,
)


@pytest.fixture
@pytest.mark.django_db
def zwarte_z_dyscyplinami(
    wydawnictwo_zwarte,
    autor_jan_nowak,
    autor_jan_kowalski,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    charaktery_formalne,
    wydawca,
    typy_odpowiedzialnosci,
    rok,
) -> Wydawnictwo_Zwarte:
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_nowak, dyscyplina_naukowa=dyscyplina1, rok=rok
    )
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski, dyscyplina_naukowa=dyscyplina2, rok=rok
    )
    wydawnictwo_zwarte.dodaj_autora(
        autor_jan_nowak, jednostka, dyscyplina_naukowa=dyscyplina1
    )

    wydawnictwo_zwarte.dodaj_autora(
        autor_jan_kowalski, jednostka, dyscyplina_naukowa=dyscyplina2
    )

    # domyslnie: ksiazka/autorstwo/wydawca spoza wykazu
    wydawnictwo_zwarte.punkty_kbn = 20
    wydawnictwo_zwarte.wydawca = wydawca
    wydawnictwo_zwarte.charakter_formalny = Charakter_Formalny.objects.get(skrot="KSP")
    wydawnictwo_zwarte.save()

    return wydawnictwo_zwarte


@pytest.fixture
@pytest.mark.django_db
def ciagle_z_dyscyplinami(
    wydawnictwo_ciagle,
    autor_jan_nowak,
    autor_jan_kowalski,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    typy_odpowiedzialnosci,
    rok,
) -> Wydawnictwo_Ciagle:
    # Przypisz autorów do dyscyplin na ten rok albo będzie awaria:
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_nowak, dyscyplina_naukowa=dyscyplina1, rok=rok
    )
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski, dyscyplina_naukowa=dyscyplina2, rok=rok
    )

    wydawnictwo_ciagle.dodaj_autora(
        autor_jan_nowak, jednostka, dyscyplina_naukowa=dyscyplina1
    )

    wydawnictwo_ciagle.dodaj_autora(
        autor_jan_kowalski, jednostka, dyscyplina_naukowa=dyscyplina2
    )

    return wydawnictwo_ciagle


@pytest.fixture
def charakter_referat():
    return Charakter_Formalny.objects.get_or_create(
        nazwa="referat zjazdowy",
        skrot="refz",
        charakter_sloty=const.CHARAKTER_SLOTY_REFERAT,
    )[0]


@pytest.fixture
def referat_z_dyscyplinami(zwarte_z_dyscyplinami, charakter_referat):
    zwarte_z_dyscyplinami.charakter_formalny = charakter_referat
    zwarte_z_dyscyplinami.save()

    return zwarte_z_dyscyplinami



from django.template.response import TemplateResponse

from .core import Komparator, StatusPorownania
from .forms import PobierzZCrossrefAPIForm


def pobierz_z_crossref(request, context=None, crossref_templates=None):
    if context is None:
        context = {}

    form = PobierzZCrossrefAPIForm()
    if request.method == "POST":
        form = PobierzZCrossrefAPIForm(request.POST)
        if form.is_valid():
            json_data = form.cleaned_data["json_data"]

            dane_porownania = Komparator.utworz_dane_porownania(json_data)

            dane_porownania_dict = {x["orig_atrybut"]: x for x in dane_porownania}

            rekord_po_stronie_bpp = Komparator.czy_rekord_ma_odpowiednik_w_bpp(
                dane_porownania_dict
            )

            context["dane_porownania"] = dane_porownania
            context["do_skopiowania"] = Komparator.dane_do_skopiowania(json_data)
            context["ignorowane"] = Komparator.dane_ignorowane(json_data)
            context["obce"] = Komparator.dane_obce(json_data)

            # Ustalamy zrodlo
            container_title = dane_porownania_dict.get("container-title", {})
            short_container_title = dane_porownania_dict.get(
                "short-container-title", {}
            )
            issn = dane_porownania_dict.get("ISSN", {})

            if container_title and short_container_title:
                if (
                    container_title["rezultat"].status == StatusPorownania.BLAD
                    and short_container_title["rezultat"].status
                    == StatusPorownania.BLAD
                ):

                    context["dane_nowego_zrodla"] = {
                        "nazwa": container_title.get("wartosc_z_crossref", ""),
                        "skrot": short_container_title.get("wartosc_z_crossref", ""),
                        "issn": issn.get("wartosc_z_crossref", ""),
                    }

            context["rekord_po_stronie_bpp"] = rekord_po_stronie_bpp

            context["identyfikator_doi"] = form.cleaned_data["identyfikator_doi"]

            return TemplateResponse(
                request, crossref_templates.get("show", ""), context
            )

    context["crossref_form"] = form
    return TemplateResponse(request, crossref_templates.get("form", ""), context)



# -*- encoding: utf-8 -*-

from django.core.management import BaseCommand

from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle, parsed_informacje_regex


class Command(BaseCommand):
    help = "Weryfikuje regex abstract.parsed_informacje_regex dla pola 'Informacje' wydawnictw ciągłych"

    def handle(self, *args, **options):
        q = Wydawnictwo_Ciagle.objects.exclude(informacje="").exclude(informacje=None).filter(rok__gte=2010)
        total = q.count()
        ret = []
        cnt = 0
        for elem in q.only("pk", "tytul_oryginalny", "informacje"):
            parsed = parsed_informacje_regex.match(elem.informacje)
            if parsed is None or parsed.groupdict()['rok'] is None:
                print(elem.pk, "|", elem.informacje, "|", elem.tytul_oryginalny)



from ..models.seria_wydawnicza import Seria_Wydawnicza
from .core import BaseBppAdminMixin

from django.contrib import admin


class Seria_WydawniczaAdmin(BaseBppAdminMixin, admin.ModelAdmin):
    list_display = [
        "nazwa",
    ]
    search_fields = [
        "nazwa",
    ]


admin.site.register(Seria_Wydawnicza, Seria_WydawniczaAdmin)



# Generated by Django 3.0.14 on 2022-06-15 19:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0319_aktualna_jednostka_to_podstawowe_miejsce'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='autor',
            name='aktualny',
        ),
    ]



# Generated by Django 3.0.14 on 2021-11-02 19:14

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("rozbieznosci_dyscyplin", "0011_null_is_wrong"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql(
                "0012_rozbieznosci_dyscyplin_zrodel", app_name="rozbieznosci_dyscyplin"
            )
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-23 08:28
from __future__ import unicode_literals

import bpp.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0109_rekordview'),
    ]

    operations = [
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_praca_recenzowana',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='always', max_length=50, verbose_name='Pokazuj opcję "Praca recenzowana"'),
        ),
    ]



from contextlib import ExitStack as does_not_raise
from decimal import Decimal

import pytest
from django.core.exceptions import ValidationError


@pytest.mark.django_db
@pytest.mark.parametrize(
    "kwargs,expectation",
    [
        (
            {"opl_pub_amount": Decimal(10.00), "opl_pub_cost_free": True},
            pytest.raises(ValidationError),
        ),
        (
            {"opl_pub_amount": Decimal(0.00), "opl_pub_cost_free": True},
            does_not_raise(),
        ),
        (
            {"opl_pub_amount": Decimal(10.00), "opl_pub_cost_free": False},
            pytest.raises(ValidationError),
        ),
        (
            {
                "opl_pub_amount": Decimal(10.00),
                "opl_pub_cost_free": False,
                "opl_pub_research_potential": True,
            },
            does_not_raise(),
        ),
        (
            {
                "opl_pub_amount": Decimal(10.00),
                "opl_pub_cost_free": False,
                "opl_pub_research_or_development_projects": True,
            },
            does_not_raise(),
        ),
        (
            {
                "opl_pub_amount": Decimal(10.00),
                "opl_pub_cost_free": False,
                "opl_pub_other": True,
            },
            does_not_raise(),
        ),
        (
            {
                "opl_pub_amount": None,
                "opl_pub_cost_free": False,
                "opl_pub_other": True,
            },
            pytest.raises(ValidationError),
        ),
    ],
)
@pytest.mark.parametrize(
    "obj",
    [
        pytest.lazy_fixture("wydawnictwo_ciagle"),
        pytest.lazy_fixture("wydawnictwo_zwarte"),
        pytest.lazy_fixture("praca_doktorska"),
        pytest.lazy_fixture("praca_habilitacyjna"),
    ],
)
def test_ModelZeZrodlemFinansowania_clean(obj, kwargs, expectation):
    for attr, value in kwargs.items():
        setattr(obj, attr, value)

    with expectation:
        obj.clean()



# Generated by Django 3.0.11 on 2021-03-07 11:04

import django.contrib.postgres.fields.jsonb
import django.core.serializers.json
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("import_pracownikow", "0004_auto_20210307_1110"),
    ]

    operations = [
        migrations.AddField(
            model_name="importpracownikowrow",
            name="log_zmian",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True,
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="importpracownikowrow",
            name="dane_z_xls",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True,
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="importpracownikowrow",
            name="dane_znormalizowane",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True,
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                null=True,
            ),
        ),
    ]



def full_name(o):
    module = o.__class__.__module__
    if module is None or module == str.__class__.__module__:
        return o.__class__.__name__
    return module + "." + o.__class__.__name__


def get_python_class_by_name(full_name):
    import importlib

    s_module = full_name[: full_name.rfind(".")]
    s_klass = full_name[full_name.rfind(".") + 1 :]

    py_module = importlib.import_module(s_module)
    py_klass = getattr(py_module, s_klass)

    return py_klass



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0030_auto_20150923_1351'),
    ]

    operations = [
        migrations.AddField(
            model_name='zrodlo',
            name='openaccess_licencja',
            field=models.ForeignKey(on_delete=models.CASCADE, verbose_name=b'OpenAccess: licencja', blank=True, to='bpp.Licencja_OpenAccess', null=True),
            preserve_default=True,
        ),
    ]



# -*- encoding: utf-8 -*-

from celery import Celery, signals
from django.conf import settings


@signals.setup_logging.connect
def setup_logging(**kwargs):
    """Setup logging."""


class Celery(Celery):
    def on_configure(self):
        if hasattr(settings, "RAVEN_CONFIG") and settings.RAVEN_CONFIG["dsn"]:
            import raven
            from raven.contrib.celery import register_logger_signal, register_signal

            client = raven.Client(settings.RAVEN_CONFIG["dsn"])
            register_logger_signal(client)
            register_signal(client)


app = Celery("django_bpp")
app.config_from_object("django.conf:settings")
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)



# -*- coding: utf-8 -*-


from django.db import migrations, models

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0070_auto_20170220_1757'),
    ]

    operations = [
        migrations.RunPython(lambda *args, **kw: load_custom_sql("0071_fix_liczba_znakow_wyd")),
    ]



from django.apps import AppConfig

from django.utils.translation import gettext_lazy as _


class DynamicColumnsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "dynamic_columns"
    verbose_name = _("Admin dynamic columns")



# Generated by Django 3.0.14 on 2021-09-26 05:54

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bpp", "0295_instaluj_szablony"),
    ]

    operations = [
        migrations.RunSQL(
            "CREATE UNIQUE INDEX bpp_szablondlaopisubibliograficznego_nulltest ON "
            "bpp_szablondlaopisubibliograficznego((model_id IS NULL)) WHERE model_id IS NULL",
            "DROP INDEX IF EXISTS bpp_szablondlaopisubibliograficznego_nulltest",
        ),
        migrations.RunSQL(
            "CREATE UNIQUE INDEX dbtemplates_template_name ON django_template(name)",
            "DROP INDEX IF EXISTS dbtemplates_template_name",
        ),
    ]



# Generated by Django 3.0.7 on 2020-07-27 21:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0217_grant"),
    ]

    operations = [
        migrations.AddField(
            model_name="praca_doktorska",
            name="grant",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.Grant",
            ),
        ),
        migrations.AddField(
            model_name="praca_habilitacyjna",
            name="grant",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.Grant",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_ciagle",
            name="grant",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.Grant",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_zwarte",
            name="grant",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.Grant",
            ),
        ),
    ]



# Generated by Django 3.0.4 on 2020-04-20 22:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0208_auto_20200329_1719"),
    ]

    operations = [
        migrations.AddField(
            model_name="jednostka",
            name="kolejnosc",
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AlterModelOptions(
            name="jednostka",
            options={
                "ordering": ["kolejnosc", "nazwa"],
                "verbose_name": "jednostka",
                "verbose_name_plural": "jednostki",
            },
        ),
    ]



# Generated by Django 3.0.7 on 2020-06-07 18:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0212_upowaznienie_pbn"),
        ("bpp", "0212_charakter_formalny_charakter_ogolny"),
    ]

    operations = []



from django.conf import settings


def theme_name(request):
    return {"THEME_NAME": "scss/" + settings.THEME_NAME + ".css"}


def enable_new_reports(request):
    return {"ENABLE_NEW_REPORTS": settings.ENABLE_NEW_REPORTS}


def max_no_authors_on_browse_jednostka_page(request):
    return {
        "MAX_NO_AUTHORS_ON_BROWSE_JEDNOSTKA_PAGE": settings.MAX_NO_AUTHORS_ON_BROWSE_JEDNOSTKA_PAGE
    }



# Generated by Django 3.0.11 on 2021-01-24 23:50

import uuid
from decimal import Decimal

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import bpp.models.uczelnia
import bpp.util


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0235_auto_20210125_0042"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("raport_slotow", "0003_auto_20200329_1719"),
    ]

    operations = [
        migrations.CreateModel(
            name="RaportSlotowUczelnia",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("last_updated_on", models.DateTimeField(auto_now=True)),
                ("started_on", models.DateTimeField(blank=True, null=True)),
                ("finished_on", models.DateTimeField(blank=True, null=True)),
                ("finished_successfully", models.BooleanField(default=False)),
                ("traceback", models.TextField(blank=True, null=True)),
                ("od_roku", models.IntegerField(default=bpp.util.year_last_month)),
                (
                    "do_roku",
                    models.IntegerField(
                        default=bpp.models.uczelnia.UczelniaManager.do_roku_default
                    ),
                ),
                (
                    "slot",
                    models.DecimalField(
                        decimal_places=4, default=Decimal("1"), max_digits=7
                    ),
                ),
                (
                    "minimalny_pk",
                    models.DecimalField(decimal_places=2, default=0, max_digits=5),
                ),
                (
                    "dziel_na_jednostki_i_wydzialy",
                    models.BooleanField(
                        default=True, verbose_name="Dziel na jednostki i wydziały"
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "ordering": ["-last_updated_on"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="RaportSlotowUczelniaWiersz",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "pkd_aut_sum",
                    models.DecimalField(
                        decimal_places=4,
                        max_digits=8,
                        verbose_name="Suma punktów dla autora",
                    ),
                ),
                ("slot", models.DecimalField(decimal_places=4, max_digits=8)),
                (
                    "autor",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="bpp.Autor"
                    ),
                ),
                (
                    "dyscyplina",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="bpp.Dyscyplina_Naukowa",
                    ),
                ),
                (
                    "jednostka",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="bpp.Jednostka"
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="raport_slotow.RaportSlotowUczelnia",
                    ),
                ),
            ],
        ),
    ]



import pytest
from django.urls import reverse

from bpp import const


@pytest.mark.vcr
@pytest.mark.parametrize(
    "identyfikator_doi,nazwa_pola,wartosc",
    [
        (
            "10.12775/jehs.2022.12.07.045",
            "tytul_oryginalny",
            "Neurological and neuropsychological post-covid complications",
        ),
        ("10.2478/cpp-2022-0006", "jezyk", "2"),
    ],
)
@pytest.mark.parametrize("url", ["wydawnictwo_ciagle"])
def test_integracyjny_strona_admina(
    url, identyfikator_doi, admin_app, jezyki, charaktery_formalne, nazwa_pola, wartosc
):
    url = (
        reverse(f"admin:bpp_{url}_add")
        + f"?{const.CROSSREF_API_PARAM}={identyfikator_doi}"
    )

    page = admin_app.get(url)

    assert page.forms[1][nazwa_pola].value == wartosc



# -*- coding: utf-8 -*-


from django.db import models, migrations
import bpp.fields


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0031_zrodlo_openaccess_licencja'),
    ]

    operations = [
        migrations.AddField(
            model_name='patent',
            name='public_dostep_dnia',
            field=models.DateField(help_text=b'Data wolnego dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='patent',
            name='public_www',
            field=models.URLField(max_length=2048, null=True, verbose_name=b'Adres WWW (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='praca_doktorska',
            name='public_dostep_dnia',
            field=models.DateField(help_text=b'Data wolnego dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='praca_doktorska',
            name='public_www',
            field=models.URLField(max_length=2048, null=True, verbose_name=b'Adres WWW (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='praca_doktorska',
            name='pubmed_id',
            field=models.BigIntegerField(help_text=b'Identyfikator PubMed (PMID)', null=True, verbose_name=b'PubMed ID', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='public_dostep_dnia',
            field=models.DateField(help_text=b'Data wolnego dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='public_www',
            field=models.URLField(max_length=2048, null=True, verbose_name=b'Adres WWW (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='pubmed_id',
            field=models.BigIntegerField(help_text=b'Identyfikator PubMed (PMID)', null=True, verbose_name=b'PubMed ID', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='public_dostep_dnia',
            field=models.DateField(help_text=b'Data wolnego dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='public_www',
            field=models.URLField(max_length=2048, null=True, verbose_name=b'Adres WWW (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='pubmed_id',
            field=models.BigIntegerField(help_text=b'Identyfikator PubMed (PMID)', null=True, verbose_name=b'PubMed ID', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='public_dostep_dnia',
            field=models.DateField(help_text=b'Data wolnego dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='public_www',
            field=models.URLField(max_length=2048, null=True, verbose_name=b'Adres WWW (wolny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='pubmed_id',
            field=models.BigIntegerField(help_text=b'Identyfikator PubMed (PMID)', null=True, verbose_name=b'PubMed ID', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='zrodlo',
            name='doi',
            field=bpp.fields.DOIField(max_length=2048, blank=True, help_text=b'Digital Object Identifier (DOI)', null=True, verbose_name=b'DOI', db_index=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='patent',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='patent',
            name='www',
            field=models.URLField(max_length=1024, null=True, verbose_name=b'Adres WWW (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='www',
            field=models.URLField(max_length=1024, null=True, verbose_name=b'Adres WWW (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='www',
            field=models.URLField(max_length=1024, null=True, verbose_name=b'Adres WWW (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='www',
            field=models.URLField(max_length=1024, null=True, verbose_name=b'Adres WWW (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='www',
            field=models.URLField(max_length=1024, null=True, verbose_name=b'Adres WWW (p\xc5\x82atny dost\xc4\x99p)', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='www',
            field=models.URLField(db_index=True, max_length=1024, null=True, verbose_name=b'WWW', blank=True),
            preserve_default=True,
        ),
    ]



API_BRAK_PARAMETRU = "Brak parametru do przeprowadzenia zapytania. "

PUBMED_PO_TYTULE_BRAK = "Szukając po tytule, nie znaleziono żadnych rekordów. "
PUBMED_PO_TYTULE_WIELE = "Znaleziono więcej, niz 1 rekord. "
PUBMED_BRAK_PARAMETRU = API_BRAK_PARAMETRU


# Używane przez testy:
PUBMED_TITLE_NONEXISTENT = "__nonexistent__"
PUBMED_TITLE_MULTIPLE = "__multiple__"



# Generated by Django 2.1.7 on 2019-09-02 23:03

import bpp.models.cache
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('rozbieznosci_dyscyplin', '0002_rok_2017_i_wyzej'),
    ]

    operations = [
        migrations.CreateModel(
            name='BrakPrzypisaniaView',
            fields=[
                ('id', bpp.models.cache.TupleField(base_field=models.IntegerField(), primary_key=True, serialize=False, size=3)),
                ('rok', models.IntegerField()),
            ],
            options={
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='RozbieznePrzypisaniaView',
            fields=[
                ('id', bpp.models.cache.TupleField(base_field=models.IntegerField(), primary_key=True, serialize=False, size=3)),
                ('rok', models.IntegerField()),
            ],
            options={
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='RozbieznosciView',
            fields=[
                ('id', bpp.models.cache.TupleField(base_field=models.IntegerField(), primary_key=True, serialize=False, size=3)),
                ('rok', models.IntegerField()),
            ],
            options={
                'managed': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-12-18 23:37
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0150_auto_20181125_1202'),
        ('bpp', '0151_snip'),
    ]

    operations = [
    ]



import itertools
from collections import OrderedDict
from decimal import Decimal
from enum import Enum
from typing import Any, Dict, List, Optional, Union

import openpyxl.worksheet.worksheet
from openpyxl.utils import get_column_letter
from openpyxl.worksheet.table import TableColumn
from unidecode import unidecode

from django.contrib.sites.models import Site

from django.utils.functional import cached_property

from bpp.util import worksheet_columns_autosize, worksheet_create_table


def chunker(n, iterable):
    iterable = iter(iterable)
    while True:
        x = tuple(itertools.islice(iterable, n))
        if not x:
            return
        yield x


class SHUFFLE_TYPE(Enum):
    BEGIN = 1
    MIDDLE = 2
    END = 3
    RANDOM = 4


import random


def shuffle_array(
    array, start, length, no_shuffles=1, shuffle_type=SHUFFLE_TYPE.MIDDLE
):

    first = array[:start]
    second = array[start : start + length]
    third = array[start + length :]

    i = shuffle_type
    if shuffle_type == SHUFFLE_TYPE.RANDOM:
        i = random.randint(1, 3)

    if i == SHUFFLE_TYPE.BEGIN:
        for a in range(no_shuffles):
            random.shuffle(first)
    elif i == SHUFFLE_TYPE.MIDDLE:
        for a in range(no_shuffles):
            random.shuffle(second)
    elif i == SHUFFLE_TYPE.END:
        for a in range(no_shuffles):
            random.shuffle(third)

    return first + second + third


def output_table_to_xlsx(
    ws: openpyxl.worksheet.worksheet.Worksheet,
    title: str,
    headers: List[str],
    dataset: List[List[Any]],
    totals: List[str] = None,
    first_column_url: str = "https://{site_name}/bpp/rekord/",
    autor_column_url: Optional[int] = 4,
    autor_column_link: Optional[int] = 1,
    autosize_columns: List[str] = None,
    column_widths: Dict[int, int] = None,
):
    ws.append(headers)

    table_columns = tuple(
        TableColumn(id=h, name=header) for h, header in enumerate(headers, start=1)
    )

    footer_row = []
    if totals:
        for no, elem in enumerate(table_columns):
            if no == 0:
                footer_row.append("Suma")
                total_column = table_columns[0]
                total_column.totalsRowLabel = footer_row[0]
                continue

            if elem.name in totals:
                count_column = table_columns[no]
                count_column.totalsRowFunction = "sum"
                footer_row.append(f"=SUBTOTAL(109,{title}[{elem.name}])")
                continue

            footer_row.append("")

    for cell in ws[ws.max_row : ws.max_row]:
        cell.font = openpyxl.styles.Font(bold=True)

    first_table_row = ws.max_row

    site_name = Site.objects.first().domain
    url = first_column_url.format(site_name=site_name)
    autor_url = f"https://{site_name}/bpp/autor/"
    for row in dataset:
        ws.append(row)

        # URL dla pierwszej kolumny -- odnośnik do BPP
        id_for_url = row[0].replace(" ", "")
        if id_for_url.startswith("("):
            id_for_url = id_for_url[1:-1]
        ws.cell(row=ws.max_row, column=1).value = '=HYPERLINK("{}", "{}")'.format(
            url + id_for_url + "/", row[0]
        )

        if autor_column_link is not None:
            # Druga kolumna z ID autora -> bpp
            ws.cell(
                row=ws.max_row, column=autor_column_link + 1
            ).value = '=HYPERLINK("{}", "{}")'.format(
                autor_url + str(row[autor_column_link]), row[autor_column_link]
            )

        if autor_column_url is not None:
            # URL dla czwartej kolumny -- odnośnik do pliku autora
            ws.cell(
                row=ws.max_row, column=autor_column_url + 1
            ).value = '=HYPERLINK("{}", "{}")'.format(
                string2fn(row[autor_column_url]) + ".xlsx", row[autor_column_url]
            )

    if footer_row:
        ws.append(footer_row)

    if dataset:
        worksheet_create_table(
            ws,
            title=title,
            first_table_row=first_table_row,
            totals=totals,
            table_columns=table_columns,
        )

    if totals is None:
        totals = []
    for elem in totals:
        letter = get_column_letter(headers.index(elem) + 1)
        for row in range(2, ws.max_row):
            ws[f"{letter}{row}"].number_format = "#,####0.0000"

        # Ustaw automatyczny rozmiar
        ws.column_dimensions[letter].bestFit = True

    dont_resize_those_columns = []
    for ncol, col in enumerate(ws.columns):
        if headers[ncol] in totals:
            dont_resize_those_columns.append(ncol)

    worksheet_columns_autosize(ws, dont_resize_those_columns=dont_resize_those_columns)


def string2fn(s):
    return s.strip().replace(" ", "_").replace("*", "x").replace("-", "_")


def autor2fn(autor):
    return string2fn(f"{autor.nazwisko}_{autor.imiona}".lower())


def normalize_xlsx_header_column_name(s):
    if s is None:
        return
    s = str(s).replace(".", " ").replace("-", " ").replace("/", " ").strip()
    if not s:
        return

    while s.find("  ") >= 0:
        s = s.replace("  ", " ")

    return unidecode(s.strip()).replace(" ", "_").lower()


def find_header_row(
    worksheet: openpyxl.worksheet.worksheet.Worksheet,
    header_row: List[str],
    max_header_row=100,
) -> Union[None, int]:
    """
    Poszukuje wierwsza nagłówka w skoroszycie ``worksheet``.

    :param max_header_row: maksymalny wiersz, w którym poszukujemy nagłówka
    :return: wiersza nagłówkowego - jeżeli znaleziony
    """

    normalized_header_row = [normalize_xlsx_header_column_name(v) for v in header_row]
    max_header_row = min(worksheet.max_row, max_header_row)

    for nrow, row in enumerate(worksheet[1:max_header_row], start=1):
        normalized_cell_values = [
            normalize_xlsx_header_column_name(cell.value)
            for cell in row[: len(header_row)]
        ]
        if normalized_cell_values == normalized_header_row:
            return nrow


class InputXLSX:
    def __init__(self, fn, header_cols):
        self.fn = fn
        self.header_cols = header_cols

    @cached_property
    def workbook(self):
        return openpyxl.load_workbook(self.fn)

    @cached_property
    def worksheet(self):
        return self.workbook.worksheets[0]

    @cached_property
    def header_normalized(self):
        return [normalize_xlsx_header_column_name(col) for col in self.header_cols]

    @cached_property
    def header_row(self):
        return find_header_row(self.worksheet, self.header_cols)

    def rows_as_list(self):
        if self.header_row is None:
            raise ValueError("Brak wiersza nagłówka w pliku XLS.")

        min_row = self.header_row + 1
        max_row = self.worksheet.max_row
        for row in self.worksheet[min_row:max_row]:
            yield [cell.value for cell in row]

    def rows_as_dict(self):
        keys = self.header_normalized[:]
        keys.append("__nrow__")

        for nrow, row in enumerate(self.rows_as_list(), start=self.header_row + 1):
            row = OrderedDict(zip(keys, row))
            row["__nrow__"] = nrow
            yield row


def float_or_string_or_int_or_none_to_decimal(i, decimal_places=4):
    if i is None:
        return i
    if type(i) in [int, Decimal, str]:
        return Decimal(i)
    if isinstance(i, float):
        return Decimal(f"%.{decimal_places}f" % round(i, decimal_places))
    raise NotImplementedError(f"Type {type(i)} not supported.")



from zope.interface import interface


class IWebTask(interface.Interface):
    """A task, that can be displayed on a web UI -- contains attributes and
    methods, that allow rendering of user-readable output.
    """
    title = interface.Attribute("Title of this task")
    slug = interface.Attribute("Slug")
    readable_arguments = interface.Method(
        "Get string containing readable arguments")


class IReportMaker(interface.Interface):
    """Something, that produces reports.
    """

    perform = interface.Method("""
        Function to run to produce the report.""")


class IReport(interface.Interface):
    """A long-running background task class, with description, title and
    parameters.
    """
    function = interface.Attribute("Function of this report")
    arguments = interface.Attribute("Arguments of this report")




from collections import namedtuple
from hashlib import md5

from asgiref.sync import async_to_sync

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.messages import DEFAULT_TAGS


def get_channel_name_for_user(user: User):
    return md5(
        str(user.pk).encode("utf-8") + str(user.username).encode("utf-8")
    ).hexdigest()


def convert_obj_to_channel_name(original):
    ctype = ContentType.objects.get_for_model(original)
    pk = original.pk
    return f"{ctype.app_label}.{ctype.model}-{pk}"


def get_obj_from_channel_name(name):
    app_label_model, pk = name.split("-", 1)
    app_label, model = app_label_model.split(".", 1)
    ctype = ContentType.objects.get(app_label=app_label, model=model)
    obj = ctype.get_object_for_this_type(pk=int(pk))
    return obj


Message = namedtuple(
    "Message", "text cssClass clickURL hideCloseOption closeURL closeText"
)
Message.__new__.__defaults__ = ("info", None, False, None, "&times;")

from channels.layers import get_channel_layer


def _send(channel_name, data):
    channel_layer = get_channel_layer()

    fun = channel_layer.group_send
    if channel_name.startswith("specific"):
        fun = channel_layer.send

    data["type"] = "chat_message"

    return async_to_sync(fun)(channel_name, data)


def send_notification(
    request_or_channel_name,
    level,
    text,
    closeURL=None,
):
    channel_name = request_or_channel_name
    if hasattr(request_or_channel_name, "user") and hasattr(
        request_or_channel_name.user, "username"
    ):
        channel_name = get_channel_name_for_user(request_or_channel_name.user)

    return _send(
        channel_name,
        Message(
            text=text, cssClass=DEFAULT_TAGS.get(level), closeURL=closeURL
        )._asdict(),
    )


def send_redirect(specific_channel, redirect_url):
    """
    :param specific_channel: nazwa kanału django-channels
    :param redirect_url: URL do którego odesłać przeglądarkę użytkownika
    :return:
    """

    return _send(specific_channel, dict(url=redirect_url))



from datetime import date

from django.test import TestCase
from model_bakery import baker

from bpp.imports.egeria_2012 import (
    dopasuj_autora,
    dopasuj_jednostke,
    importuj_wiersz,
    mangle_labels,
    znajdz_lub_zrob_stanowisko,
)
from bpp.imports.uml import UML_Egeria_2012_Mangle
from bpp.models import Autor, Autor_Jednostka, Funkcja_Autora
from bpp.tests.util import any_jednostka


class _x:
    def __init__(self, value):
        self.value = value


class TestImports(TestCase):
    def test_mangle_labels(self):
        a = [
            "foo",
            "bar",
            "baz",
            "quux",
            "Stanowisko",
            "foo",
            "bar",
            "quux",
            "Stanowisko",
            "Wydzial ",
            "Stanowisko",
            "Wydzial ",
            "Stanowisko",
            "Wydzial",
        ]

        b = [
            "foo",
            "bar",
            "baz",
            "quux",
            "Stanowisko",
            "foo",
            "bar",
            "quux",
            "Stanowisko_2009",
            "Wydzial_2009",
            "Stanowisko_2010",
            "Wydzial_2010",
            "Stanowisko_2011",
            "Wydzial_2011",
        ]

        self.assertEqual(mangle_labels(a), b)

    def test_znajdz_lub_zrob_stanowisko(self):
        f = znajdz_lub_zrob_stanowisko("Kucharz")
        self.assertEqual(f.nazwa, "Kucharz")
        f1 = znajdz_lub_zrob_stanowisko("Kucharz")
        self.assertEqual(f1, f)

    def test_dopasuj_jednostke(self):
        mangle = UML_Egeria_2012_Mangle
        j1 = any_jednostka(nazwa="Foo")
        j2 = any_jednostka(  # noqa
            nazwa="Sam. Pracownia Propedeutyki Radiologii Stom. i Szczęk-Twarz"
        )
        j3 = any_jednostka(
            nazwa="Katedra i Klinika Ginekologii i Endokrynologii Ginekologicznej"
        )
        j4 = any_jednostka(nazwa="I Katedra i Klinika Ginekologii")
        j5 = any_jednostka(nazwa="II Katedra i Klinika Ginekologii")  # noqa

        self.assertEqual(
            dopasuj_jednostke(
                "Katedra i Klinika Ginekologii i Endokrynologii Ginekolog.", mangle
            ),
            j3,
        )

        self.assertEqual(dopasuj_jednostke("Foo", mangle), j1)

        self.assertEqual(
            dopasuj_jednostke(
                "Sam. Pracownia Propedeutyki Radiologii Stom. i Szczęk-Twarz", mangle
            ),
            None,
        )

        self.assertEqual(
            dopasuj_jednostke("I Katedra i Klinika Ginekologii", mangle), j4
        )

    def test_dopasuj_autora(self):
        a1 = baker.make(Autor, imiona="Jan", nazwisko="Kowalski")
        a2 = baker.make(Autor, imiona="Jan", nazwisko="Kowalski")
        a3 = baker.make(Autor, imiona="Jan", nazwisko="Unikalny")

        j1 = any_jednostka()
        j2 = any_jednostka()
        f1 = baker.make(Funkcja_Autora, nazwa="Kucharz")

        Autor_Jednostka.objects.create(autor=a1, jednostka=j1, funkcja=f1)

        Autor_Jednostka.objects.create(autor=a2, jednostka=j2, funkcja=f1)

        self.assertEqual(dopasuj_autora("Jan", "Unikalny", None, None), a3)

        self.assertEqual(dopasuj_autora("Jan", "Kowalski", j1.nazwa, f1), a1)

        self.assertEqual(dopasuj_autora("Jan", "Kowalski", j2.nazwa, f1), a2)

    def test_importuj_wiersz(self):
        a1 = baker.make(Autor, imiona="Jan", nazwisko="Kowalski")
        j1 = any_jednostka(nazwa="Foo")

        importuj_wiersz(
            "Jan", "Kowalski", "Foo", "Kucharz", 2012, UML_Egeria_2012_Mangle
        )
        self.assertEqual(
            Autor_Jednostka.objects.get(autor=a1, jednostka=j1).zakonczyl_prace,
            date(2012, 12, 31),
        )

    # def test_importuj_sheet_roczny(self):
    #     a1 = baker.make(Autor, imiona='Jan', nazwisko='Kowalski')
    #     j1 = any_jednostka(nazwa='Foo')
    #
    #     with Mock() as sheet:
    #         sheet.row(0) >> [_x('Stanowisko'), _x('PRC IMIE'), _x('PRC NAZWISKO'), _x('OB_NAZWA')]
    #         sheet.nrows >> 1
    #         sheet.row(1) >> [_x('Kucharz'), _x('Jan'), _x('Kowalski'), _x(u'Zakład')]
    #
    #     importuj_sheet_roczny(sheet, 2012, UML_Egeria_2012_Mangle)
    #
    # def test_importuj_sheet_osoby_nie_ujete(self):
    #     w1 = any_wydzial(nazwa='Wydział Kucharski')
    #     a1 = any_autor(imiona='Jan', nazwisko='Kowalski')
    #     j1 = any_jednostka(nazwa=u'Zakład', wydzial=w1)
    #
    #     with Mock() as sheet:
    #         sheet.row(2) >> [_x("NUMER"), _x("PRC IMIE"), _x("PRC NAZWISKO"), _x("OB_NAZWA"),
    #                          _x("Stanowisko"), _x("WYMIAR ETATU"),
    #                          _x('stanowisko'), _x(u'stopień'), _x(u'Wydział'),
    #                          _x('stanowisko'), _x(u'stopień'), _x(u'Wydział'),
    #                          _x('stanowisko'), _x(u'stopień'), _x(u'Wydział'),
    #                          _x('stanowisko'), _x(u'stopień'), _x(u'Wydział')]
    #         sheet.nrows >> 4
    #         sheet.row(3) >> [_x('123'), _x('Jan'), _x('Kowalski'), _x(u'Zakład'),
    #                          _x('Kucharz'), _x('Pełny'),
    #                          _x('#N/D!'), _x('#N/D!'), _x('#N/D!'),
    #                          _x('Sternik'), _x(u'Młodzszy'), _x(u'Tego wydziału nie ma'),
    #                          _x("Kucharz"), _x('Starszy'), _x(u'Wydział Kucharski'),
    #                          _x("Kucharz"), _x('Starszy'), _x(u'Wydział Kucharski'),
    #                          ]
    #
    #     importuj_sheet_osoby_nie_ujete(sheet, UML_Egeria_2012_Mangle)
    #
    #     l = list(Opi_2012_Afiliacja_Do_Wydzialu.objects.all().order_by('rok'))
    #     self.assertEquals(len(l), 2)
    #     self.assertEquals(l[0].autor, a1)
    #     self.assertEquals(l[0].wydzial, w1)
    #     self.assertEquals(l[0].rok, 2011)
    #


#
# class TestImportujImiona(TestCase):
#
#
#     def test_importuj_imiona(self):
#
#         with Mock() as sheet:
#             sheet.nrows >> 3
#             sheet.row(0) >> [_x(a) for a in ['0', 'dr', 'Jan Maria', 'Kowalski', 'nie']]
#             sheet.row(1) >> [_x(a) for a in ['0', 'dr', 'Jan Maria', 'Nowak', 'nie']]
#             sheet.row(2) >> [_x(a) for a in ['0', 'dr', 'Jan Maria', 'Strzelec', 'nie']]
#
#         wydzial = any_wydzial()
#         wydzial2 = any_wydzial()
#
#         jednostka = any_jednostka(wydzial=wydzial)
#         jednostka2 = any_jednostka(wydzial=wydzial2)
#
#         autor = baker.make(Autor, nazwisko='Kowalski', imiona='Jan')
#         Autor_Jednostka.objects.create(jednostka=jednostka, autor=autor)
#
#         autor2 = baker.make(Autor, nazwisko='Nowak', imiona='Jan')
#         Autor_Jednostka.objects.create(jednostka=jednostka, autor=autor2)
#
#         autor3 = baker.make(Autor, nazwisko='Nowak', imiona='Jan')
#         Autor_Jednostka.objects.create(jednostka=jednostka, autor=autor3)
#
#         autor4 = baker.make(Autor, nazwisko='Strzelec', imiona='Jan')
#         Autor_Jednostka.objects.create(jednostka=jednostka, autor=autor4)
#
#         autor5 = baker.make(Autor, nazwisko='Strzelec', imiona='Jan')
#         Autor_Jednostka.objects.create(jednostka=jednostka2, autor=autor5)
#
#         importuj_imiona_sheet(sheet, wydzial)
#
#         autor = Autor.objects.get(pk=autor.pk)
#         self.assertEquals(autor.imiona, 'Jan Maria')
#
#         # A u tych dwóch nic się nie zmieni...
#         autor = Autor.objects.get(pk=autor2.pk)
#         self.assertEquals(autor.imiona, 'Jan')
#         autor = Autor.objects.get(pk=autor3.pk)
#         self.assertEquals(autor.imiona, 'Jan')
#
#         # A u tego się zmieni tylko ten który jest w Wydziale Lekarskim
#         autor = Autor.objects.get(pk=autor4.pk)
#         self.assertEquals(autor.imiona, 'Jan Maria')
#         autor = Autor.objects.get(pk=autor5.pk)
#         self.assertEquals(autor.imiona, 'Jan')



from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import JSONField

from formdefaults.core import get_form_defaults
from formdefaults.util import full_name, get_python_class_by_name


class FormRepresentationManager(models.Manager):
    def get_or_create_for_instance(self, form_instance):
        """
        Ta klasa otrzymuje jako parametr instancję formularza Django wraz
        z czytelną nazwą i ewentualnym kodem formularza (jeżeli go nie ma,
        zostanie wygenerowany automatycznie na podstawie nazwy).

        Następnie mapuje pola w tym formularzu po nazwie (w kontekście bazy
        danych) i szuka ich ewentualnych ustawień domyślnych (zakodowanych
        w formacie JSON)
        """
        fn = full_name(form_instance)
        res, created = self.get_or_create(full_name=fn)
        return res


class FormRepresentation(models.Model):
    full_name = models.TextField("Kod formularza", primary_key=True)
    label = models.TextField("Nazwa formularza")

    html_before = models.TextField("Kod HTML przed formularzem", null=True, blank=True)
    html_after = models.TextField("Kod HTML po formularzu", null=True, blank=True)

    objects = FormRepresentationManager()

    def __str__(self):
        return f"Formularz '{self.label}'"

    def get_form_class(self):
        """Zwraca klasę formularza zapisaną pod self.full_name lub None"""
        return get_python_class_by_name(self.full_name)

    class Meta:
        unique_together = [("full_name", "label")]
        verbose_name = "Lista wartości domyślnych formularza"
        verbose_name_plural = "Listy wartości domyślnych formularzy"


class FormFieldRepresentation(models.Model):
    """Default value in a form."""

    parent = models.ForeignKey(
        FormRepresentation,
        on_delete=models.CASCADE,
        related_name="fields_set",
        help_text="Formularz, do którego należy to pole",
    )

    name = models.TextField("Systemowa nazwa pola")
    label = models.TextField("Czytelna etykieta pola", null=True, blank=True)
    klass = models.TextField("Klasa pola")
    order = models.PositiveSmallIntegerField()

    class Meta:
        verbose_name = "Pole formularza"
        verbose_name_plural = "Pola formularzy"
        unique_together = [
            ("parent", "name"),
        ]
        ordering = ("order",)

    def __str__(self):
        return self.label or self.name


class FormFieldDefaultValue(models.Model):
    parent = models.ForeignKey(
        FormRepresentation,
        on_delete=models.CASCADE,
        related_name="values_set",
        help_text="Formularz, do którego należy pole, dla którego to zdefiniowana jest"
        "ta wartość domyślna pola. ",
    )

    field = models.ForeignKey(
        FormFieldRepresentation,
        verbose_name="Pole formularza",
        on_delete=models.CASCADE,
        related_name="+",
        help_text="Pole formularza, dla którego zdefiniowana jest ta wartość domyślna. ",
    )
    value = JSONField("Wartość", null=True, blank=True)

    # Ustawienie może dotyczyć wyłącznie konkretnego użytkownika lub
    # wszystkich użytkowników systemu
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name="Użytkownik",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
    )

    def clean(self):
        # Rodzic musi istnieć po obydwu stronach, tzn po stronie FormFieldDefaultValue
        # oraz po stronie FormFieldRepresentation
        if self.parent_id is None or (
            self.field_id is not None and self.field.parent is None
        ):
            raise ValidationError(
                "Rodzic dla reprezentacji pola i rodzic dla wartości domyślnej "
                "powinien być określony w bazie danych. Skontaktuj się z administratorem "
                "systemu, jezeli dostajesz ten błąd. "
            )

        if self.parent_id != self.field.parent_id:
            raise ValidationError(
                "Rodzic wartości domyślnej i rodzic wybranego pola musi być identyczny"
            )

        # Tu potrzebujemy zaimplementować: zebranie wartości domyślnych dla w/wym formularza
        # dla self.user, następnie zainicjowanie formularza z tymi parametrami i ewentualną
        # próbę wyrenderowania go. W sytuacji, gdy zostanie podany poprawny JSON ale niepoprawna
        # wartość dla danego pola (np wartość dla pola ID jako liczba zmiennoprzecinkowa a nie
        # całkowita), użytkownik powinien otrzymać adekwatny błąd"""

        try:
            klass = self.parent.get_form_class()
        except Exception:
            raise ValidationError(
                f"Nie udało się odnaleźć klasy formularza w kodzie dla '{self.parent.full_name}'. "
                f"Weryfikacja wartości domyślnych formularza nie jest możliwa, a co za tym idzie, "
                f"zapis zmian również. Skontaktuj się z administratorem systemu. "
            )

        initial = get_form_defaults(klass(), user=self.user, update_db_repr=False)
        initial[self.field.name] = self.value

        instance = klass(initial=initial)
        try:
            form_field = instance.fields[self.field.name]
            form_field.required = False

            form_field.to_python(self.value)
            form_field.validate(self.value)
            instance.as_p()
        except Exception as e:
            raise ValidationError(
                "Nie udało się wyrenderować formularza z wpisanymi przykładowymi informacjami. "
                f"Komunikat błędu: {str(e)}. Proszę, skoryguj daną wartośc i spróbuj ponownie, "
                "ewentualnie skontaktuj się z administratorem. "
            )

    class Meta:
        verbose_name = "Wartość domyslna dla pola formularza"
        verbose_name_plural = "Wartości domyślne dla pól formularzy"
        ordering = ("user", "field__order")



import pytest

from bpp.views.raporty import RankingAutorow


@pytest.mark.django_db
def test_ranking_autorow_get_queryset_prace_sa(
    wydawnictwo_ciagle_z_autorem,
    wydawnictwo_zwarte_z_autorem,
    rf,
    uczelnia,
    przed_korekta,
):
    wydawnictwo_ciagle_z_autorem.punkty_pk = 20
    wydawnictwo_ciagle_z_autorem.impact_factor = 20
    wydawnictwo_ciagle_z_autorem.status_korekty = przed_korekta
    wydawnictwo_ciagle_z_autorem.save()

    wydawnictwo_zwarte_z_autorem.impact_factor = 50
    wydawnictwo_zwarte_z_autorem.punkty_pk = 20
    wydawnictwo_zwarte_z_autorem.status_korekty = przed_korekta
    wydawnictwo_ciagle_z_autorem.save()

    r = RankingAutorow(request=rf.get("/"), kwargs=dict(od_roku=0, do_roku=3030))

    assert r.get_queryset().count() == 1

    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)

    assert r.get_queryset().count() == 0


@pytest.mark.django_db
def test_ranking_autorow_po_typie_kbn(
    wydawnictwo_ciagle_z_autorem,
    rf,
):
    wydawnictwo_ciagle_z_autorem.punkty_pk = 20
    wydawnictwo_ciagle_z_autorem.impact_factor = 20
    wydawnictwo_ciagle_z_autorem.save()

    r = RankingAutorow(request=rf.get("/"), kwargs=dict(od_roku=0, do_roku=3030))

    assert r.get_queryset().count() == 1

    tk = wydawnictwo_ciagle_z_autorem.typ_kbn
    tk.wliczaj_do_rankingu = False
    tk.save()

    r = RankingAutorow(request=rf.get("/"), kwargs=dict(od_roku=0, do_roku=3030))

    assert r.get_queryset().count() == 0


@pytest.mark.django_db
def test_ranking_autorow_po_charakterze_formalnym(
    wydawnictwo_ciagle_z_autorem,
    rf,
):
    wydawnictwo_ciagle_z_autorem.punkty_pk = 20
    wydawnictwo_ciagle_z_autorem.impact_factor = 20
    wydawnictwo_ciagle_z_autorem.save()

    r = RankingAutorow(request=rf.get("/"), kwargs=dict(od_roku=0, do_roku=3030))

    assert r.get_queryset().count() == 1

    cf = wydawnictwo_ciagle_z_autorem.charakter_formalny
    cf.wliczaj_do_rankingu = False
    cf.save()

    r = RankingAutorow(request=rf.get("/"), kwargs=dict(od_roku=0, do_roku=3030))

    assert r.get_queryset().count() == 0



# Generated by Django 3.0.14 on 2021-08-08 10:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0020_przemapuj"),
    ]

    operations = [
        migrations.AlterField(
            model_name="scientist",
            name="lastName",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name="scientist",
            name="name",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name="scientist",
            name="orcid",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name="scientist",
            name="pbnId",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name="scientist",
            name="polonUid",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name="scientist",
            name="qualifications",
            field=models.TextField(
                blank=True, db_index=True, null=True, verbose_name="Tytuł"
            ),
        ),
    ]



from django.db import models

from import_common.core import matchuj_wydawce
from .base import BasePBNMongoDBModel

from bpp import const
from bpp.models import LinkDoPBNMixin


class PublisherManager(models.Manager):
    def official(self):
        return self.exclude(mniswId=None)


class Publisher(LinkDoPBNMixin, BasePBNMongoDBModel):
    objects = PublisherManager()

    url_do_pbn = const.LINK_PBN_DO_WYDAWCY
    atrybut_dla_url_do_pbn = "pk"

    class Meta:
        verbose_name = "Wydawca w PBN API"
        verbose_name_plural = "Wydawcy w PBN API"

    pull_up_on_save = ["publisherName", "mniswId"]

    publisherName = models.TextField(null=True, blank=True, db_index=True)
    mniswId = models.IntegerField(null=True, blank=True, db_index=True)

    def __str__(self):
        return f"{self.publisherName}, MNISW ID: {self.mniswId or '-'}"

    @property
    def points(self):
        return self.current_version["object"]["points"]

    def rekord_w_bpp(self):
        from bpp.models import Wydawca

        try:
            return Wydawca.objects.get(pbn_uid_id=self.pk)
        except Wydawca.DoesNotExist:
            pass

    def matchuj_wydawce(self):
        return matchuj_wydawce(self.publisherName, self.pk)



# -*- encoding: utf-8 -*-
from datetime import date

import os

from django.core.management import BaseCommand
from django.db import transaction

from bpp.models import Autor_Jednostka

from bpp.imports.egeria_2012 import importuj_afiliacje
from bpp.imports.uml import UML_Egeria_2012_Mangle
from bpp.management.commands import files_or_directory
from bpp.models.cache import Rekord
from bpp.models.system import Jezyk
from bpp.views.api.pubmed import get_data_from_ncbi, parse_data_from_ncbi


class Command(BaseCommand):
    help = 'Pobiera PubMed IDs dla wszystkich angielskich publikacji 2013-2015'

    def handle(self, *args, **options):
        for rec in Rekord.objects.filter(jezyk=Jezyk.objects.get(skrot='ang.'),
                                         rok__in=[2013,2014,2015]):
            orig = rec.original
            if orig.pubmed_id:
                continue

            print("Pobieram: %r" % orig.tytul_oryginalny)

            xml = get_data_from_ncbi(orig.tytul_oryginalny)
            if len(xml) == 1:
                data = parse_data_from_ncbi(xml[0])
                print(data)

                changed = False

                if data.get('pubmed_id'):
                    orig.pubmed_id = data.get('pubmed_id')
                    changed = True

                if data.get('doi'):
                    orig.doi = data.get('doi')
                    changed = True

                if changed:
                    orig.save()


import sys
import traceback
from contextlib import redirect_stderr, redirect_stdout

from django.core.management import ManagementUtility
from sentry_sdk import capture_exception

from tee.models import Log
from tee.utils import TeeIO

from django.utils import timezone


def execute(argv, **kwargs):
    """This function calls django command specified as ``command_name``,
    saving it's stdout and stderr to an object ``tee.models.Log``, also
    outputting it on stdout/stderr.

    This way one can transparently save output to the database and also
    print it out on the console. This could be useful for cron scripts."""
    stdout = TeeIO(kwargs.get("stdout") if "stdout" in kwargs else sys.stdout)
    stderr = TeeIO(kwargs.get("stderr") if "stderr" in kwargs else sys.stdout)

    log = Log.objects.create(command_name=argv[0], args=argv[1:])

    # TODO: this is memory-inefficient. stdout or stderr should flush from time to time
    # and save the result to the database, like in 1-MB chunks

    try:
        with redirect_stderr(stderr):
            with redirect_stdout(stdout):
                try:
                    utility = ManagementUtility(argv)
                    utility.execute()
                    log.finished_successfully = True
                except Exception as e:
                    capture_exception(e)

                    log.finished_successfully = False
                    log.traceback = traceback.format_exc(limit=65535)

    finally:
        log.stdout = stdout.getvalue()
        log.stderr = stderr.getvalue()
        log.finished_on = timezone.now()
        log.save()



import pytest

from zglos_publikacje.models import Zgloszenie_Publikacji


@pytest.fixture
def zgloszenie_publikacji(
    typy_odpowiedzialnosci, autor_jan_kowalski, jednostka, rok
) -> Zgloszenie_Publikacji:
    z = Zgloszenie_Publikacji.objects.create(
        tytul_oryginalny="test",
        rok=rok,
        email="foo@bar.pl",
        rodzaj_zglaszanej_publikacji=Zgloszenie_Publikacji.Rodzaje.POZOSTALE,
        strona_www="https://onet.pl/",
    )

    z.zgloszenie_publikacji_autor_set.create(
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        typ_odpowiedzialnosci=typy_odpowiedzialnosci["aut."],
        rok=rok,
    )

    return z


@pytest.fixture
def zgloszenie_publikacji_z_oplata(zgloszenie_publikacji) -> Zgloszenie_Publikacji:
    zgloszenie_publikacji.opl_pub_cost_free = False
    zgloszenie_publikacji.opl_pub_amount = 20000
    zgloszenie_publikacji.opl_pub_research_potential = True
    zgloszenie_publikacji.save()
    return zgloszenie_publikacji



# -*- encoding: utf-8 -*-

from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from mock import Mock

from bpp.admin import Wydawnictwo_ZwarteAdmin
from bpp.admin.filters import CalkowitaLiczbaAutorowFilter, LiczbaZnakowFilter
from bpp.models import (
    Autor,
    Charakter_Formalny,
    Jednostka,
    Patent,
    Praca_Doktorska,
    Praca_Habilitacyjna,
    Wydawnictwo_Zwarte,
    Zrodlo,
)
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
from bpp.system import groups
from bpp.tests.tests_legacy.testutil import SuperuserTestCase, TestCase, UserTestCase
from bpp.tests.util import any_ciagle
from bpp.views.admin import WydawnictwoCiagleTozView


class TestLiczbaZnakowFilter(TestCase):
    def test_simpleIntegerFilters(self):
        for klass in [LiczbaZnakowFilter, CalkowitaLiczbaAutorowFilter]:
            flt = klass(Mock(), [], Wydawnictwo_Zwarte, Wydawnictwo_ZwarteAdmin)

            for elem in ["brak", "zero", "powyzej"]:
                flt.value = Mock(return_value=elem)
                queryset = Mock()
                flt.queryset(Mock(), queryset)
                self.assertEqual(queryset.filter.called, True)

            flt.value = Mock(return_value="__nie ma tego parametru")
            queryset = Mock()
            flt.queryset(Mock(), queryset)
            self.assertEqual(queryset.filter.called, False)


class TestNormalUserAdmin(UserTestCase):
    def test_root(self):
        self.user.is_staff = True
        self.user.is_superuser = False

        for grupa in groups:
            self.user.groups.add(Group.objects.get_by_natural_key(grupa))
        self.user.save()

        self.assertContains(
            self.client.get("/admin/"), "Administracja", status_code=200
        )


class TestAdmin(SuperuserTestCase):
    def test_root(self):
        self.assertContains(
            self.client.get("/admin/"), "Administracja", status_code=200
        )

    def test_custom_app_index(self):
        """Spowoduje wywołanie customappindex z django-admin-tools"""
        self.assertContains(
            self.client.get("/admin/bpp/"), "Użytkownicy", status_code=200
        )

    def test_wyszukiwanie(self):
        """Dla wielu różnych model spróbuj wyszukiwać w tabelce
        i zobacz, czy nie ma błędu."""
        for model in [
            Jednostka,
            Autor,
            Zrodlo,
            Wydawnictwo_Ciagle,
            Wydawnictwo_Zwarte,
            Praca_Doktorska,
            Praca_Habilitacyjna,
            Patent,
            Charakter_Formalny,
        ]:
            content_type = ContentType.objects.get_for_model(model)
            url = reverse(
                "admin:%s_%s_changelist" % (content_type.app_label, content_type.model)
            )
            res = self.client.get(url, data={"q": "wtf"})
            self.assertEqual(res.status_code, 200)


class TestAdminViews(TestCase):
    def test_wydawnictwociagletozview(self):
        c1 = any_ciagle()
        self.assertEqual(Wydawnictwo_Ciagle.objects.count(), 1)

        w = WydawnictwoCiagleTozView()
        url = w.get_redirect_url(c1.pk)

        # Czy jest poprawne ID w URLu?
        c2 = Wydawnictwo_Ciagle.objects.all().exclude(pk=c1.pk)
        self.assertIn(str(c2[0].pk), url)

        # Czy stworzył kopię?
        self.assertEqual(Wydawnictwo_Ciagle.objects.count(), 2)



from django_bpp import urls

urls


from bpp.tests.util import *  # noqa
from bpp.tests.util import setup_model_bakery

setup_model_bakery()
#
# from test_abstract import *
# from test_admin import *
# from test_autocomplete import *
# from test_cache import *
# from test_commands import *
# from test_imports import *
# from test_jezyk_polski import *
# from test_management import *
# from test_models import *
# from test_multiseek import *
# from test_tasks import *
# from test_templatetags import *
# from test_util import *
#
# from test_views import *
# from test_reports import *
#
# import os
# if os.getenv("NO_SELENIUM", "0") != "1":
#    from test_selenium import *
# from test_selenium.test_admin import *



# -*- encoding: utf-8 -*-
import argparse
import logging

from django.core.management import BaseCommand
from django.db import transaction
from openpyxl import load_workbook

from bpp.models import Zrodlo, Rodzaj_Zrodla


class Command(BaseCommand):
    help = "Rozszera skroty zrodel na podstawie pliku"

    def add_arguments(self, parser):
        parser.add_argument("plik", type=argparse.FileType("rb"))

    @transaction.atomic
    def handle(self, plik, *args, **options):
        verbosity = int(options["verbosity"])
        logger = logging.getLogger("django")
        if verbosity > 1:
            logger.setLevel(logging.DEBUG)

        wb = load_workbook(plik)

        for sheet in wb.worksheets:
            for row in list(sheet.rows):
                skrot = row[0].value
                nazwa = row[1].value
                # poprzednia_nazwa = row[2].value
                try:
                    issn = row[2].value
                except IndexError:
                    issn = None

                try:
                    e_issn = row[3].value
                except IndexError:
                    e_issn = None

                if skrot is None or nazwa is None:
                    print(
                        f"Niepoprawna linia: {[x.value for x in row]}, skrot lub nazwa sa puste"
                    )
                    continue

                try:
                    z = Zrodlo.objects.get(skrot=skrot)
                except Zrodlo.DoesNotExist:
                    print(f"Nie ma takiego zrodla jak {skrot}")
                    continue

                needs_saving = []
                if z.nazwa != nazwa:
                    z.nazwa = nazwa
                    needs_saving.append("nazwa")

                if issn is not None and z.issn != str(issn):
                    z.issn = issn
                    needs_saving.append("issn")

                if e_issn is not None and z.e_issn != str(e_issn):
                    z.e_issn = e_issn
                    needs_saving.append("e_issn")

                # if (
                #     poprzednia_nazwa is not None
                #     and z.poprzednia_nazwa != poprzednia_nazwa
                # ):
                #     z.poprzednia_nazwa = poprzednia_nazwa
                #     needs_saving.append("poprzednia")

                if needs_saving:
                    print(f"zmieniam {z.nazwa, z.pk} bo {needs_saving}")
                    z.save()



from functools import cached_property

from dynamic_columns.models import ModelAdmin

# TODO: 1) checks for list_display_always (czy kolumna istnieje)


class DynamicColumnsMixin:
    """This is a mixin you use in your ModelAdmin class.

    It enables the ModelAdmin to:

    1) automatically create its in-database representation (``ModelAdmin`` model),

    2) basing on this mixin settings, it creates a list of columns in-database
       (``ModelAdminColumn`` instances) thus giving the end-user ability to enable or disable
       them,

    3) keeps a list of "forbidden" columns, eg. columns that should never be visible.
    """

    # ``list_display_always``
    #
    # this is a list of always-displayed columns. Those columns are displayed
    # first in the list view of Django's Admin, they cannot be enabled or disabled,
    # they cannot be moved. This does what the "old" ``list_display`` did:

    # list_display_always = []

    # ``list_display_default``
    #
    # this is a list of columns enabled by default, that can be moved or
    # disabled (hidden) via admin interface at some point later. Here one can
    # give the end-user a reasonable selection of initially visible columns:

    # list_display_default = []

    # ``list_display_allowed``
    #
    # this is a list of columns not enabled by default, but they can be enabled
    # in the admin later:

    # list_display_allowed = []

    # ``list_display_forbidden``
    #
    # You can use "__all__" string in above variables if you want to give the
    # user access to all the attributes of the model to be used as columns
    # in the admin; this way user can decide what the user wants. But the
    # column selection could be too broad. We use ``list_display_forbidden``
    # to avoid some columns.
    #
    # This is a list of columns that are forbidden - those columns should not
    # appear in database, and even if they do - they should not be allowed to be
    # displayed.
    #
    # ``list_display_forbidden`` is in fact a list of regex that will be matched
    # against column names.

    # list_display_forbidden = []

    # ``list_select_related``
    #
    # It becomes a dictionary now! Unless this code becomes Django's mainstream,
    # please ignore system check ``admin.E117`` -- add the line below to ``settings.py``
    #
    #   SILENCED_SYSTEM_CHECKS = ["admin.E117"]
    #
    # Example ``list_select_related`` below:
    #
    #     list_select_related = {
    #         "__always__": ["some", "columns"],
    #         "some_other_column": [
    #             "some_other_column",
    #         ],
    #         "admin_callable": ["another_column", "maybe_more"],
    #     }

    @cached_property
    def _modeladmin_enabled(self):
        return ModelAdmin.objects.enable(self)

    def get_list_display(self, request):
        ret = self._modeladmin_enabled
        return ret.get_list_display(model_admin=self, request=request)

    def get_list_select_related(self, request):
        """
        As DynamicColumnsMixin allowed Django to display variable number of columns,
        the ``list_select_related`` attribute can be also dynamic. Why use ``select_related``
        with a column name, if that column was disabled by the end-user?

        DynamicColumnsMixin assumes, that ``list_select_related`` can be also a dictionary.
        If you define it as one, unless this code becomes Django's mainstream,
        please ignore system check ``admin.E117`` -- add the line below to ``settings.py``

            SILENCED_SYSTEM_CHECKS = ["admin.E117"]

        Example ``list_select_related`` below:

            list_select_related = {
                "__always__": ["some", "columns"],
                "some_other_column": [
                    "some_other_column",
                ],
                "admin_callable": ["another_column", "maybe_more"],
            }

        The key ``__always__`` is special -- it contains the columns that will be always
        returned by DynamicColumnsMixin.get_select_related. Think about it as
        "old list_select_related".
        """

        # If the type of self.list_select_related is a list, just return it.
        # This is standard Django's ModelAdmin behavior:
        if isinstance(self.list_select_related, (list, tuple, set)):
            return self.list_select_related

        # IF the type of self.list_select_related is a dict, this means it describes
        # a mapping between a column name (and this column may or may not be visible)
        # and it's select_related name.
        columns = self.get_list_display(request)

        ret = set()
        for elem in self.list_select_related:
            if elem in columns or elem == "__always__":
                values = self.list_select_related[elem]
                if isinstance(values, str):
                    ret.add(values)
                elif isinstance(values, (list, set, tuple)):
                    [ret.add(_x) for _x in values]
                else:
                    raise NotImplementedError(
                        f"Handling of values of type {type(values)} not implemented"
                    )
        return ret



# Generated by Django 3.0.14 on 2021-11-02 16:55

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0307_streszczenia"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0308_ewaluacja_upowaznienia_view")
        )
    ]



from rest_framework import serializers
from taggit_serializer.serializers import TagListSerializerField

from api_v1.serializers.util import (
    AbsoluteUrlSerializerMixin,
    Wydawnictwo_AutorSerializerMixin,
    WydawnictwoSerializerMixin,
)
from bpp.models import Patent, Patent_Autor


class Patent_AutorSerializer(
    Wydawnictwo_AutorSerializerMixin, serializers.HyperlinkedModelSerializer
):
    rekord = serializers.HyperlinkedRelatedField(
        view_name="api_v1:patent-detail", read_only=True
    )

    class Meta:
        model = Patent_Autor
        fields = [
            "id",
            "autor",
            "jednostka",
            "zapisany_jako",
            "typ_odpowiedzialnosci",
            "afiliuje",
            "zatrudniony",
            "kolejnosc",
            "rekord",
            "procent",
            "dyscyplina_naukowa",
        ]


class PatentSerializer(
    AbsoluteUrlSerializerMixin,
    WydawnictwoSerializerMixin,
    serializers.HyperlinkedModelSerializer,
):
    slowa_kluczowe = TagListSerializerField()

    rodzaj_prawa = serializers.RelatedField(read_only=True)
    # "bpp.Rodzaj_Prawa_Patentowego", CASCADE, null=True, blank=True
    # )

    wydzial = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydzial-detail", read_only=True
    )

    autorzy_set = serializers.HyperlinkedRelatedField(
        many=True, view_name="api_v1:wydawnictwo_zwarte_autor-detail", read_only=True
    )

    class Meta:
        model = Patent
        fields = [
            "id",
            #
            "tytul_oryginalny",
            #
            "data_zgloszenia",
            "numer_zgloszenia",
            "data_decyzji",
            "numer_prawa_wylacznego",
            "rodzaj_prawa",
            "wdrozenie",
            "wydzial",
            #
            "rok",
            "status_korekty",
            #
            "jezyk",
            "jezyk_alt",
            "charakter_formalny",
            "typ_kbn",
            #
            "www",
            "dostep_dnia",
            "public_www",
            "public_dostep_dnia",
            #
            "pbn_id",
            #
            "impact_factor",
            "punkty_kbn",
            #
            "informacje",
            "szczegoly",
            "uwagi",
            "slowa_kluczowe",
            "strony",
            "tom",
            #
            "utworzono",
            "ostatnio_zmieniony",
            #
            "absolute_url",
            #
            "tekst_przed_pierwszym_autorem",
            "autorzy_set",
            "tekst_po_ostatnim_autorze",
        ]



from django import forms

from ewaluacja2021.models import LiczbaNDlaUczelni
from ..models import Uczelnia, Ukryj_Status_Korekty, Wydzial

# Uczelnia
from .core import BaseBppAdminMixin, RestrictDeletionToAdministracjaGroupMixin
from .helpers import ADNOTACJE_FIELDSET, ZapiszZAdnotacjaMixin

from django.contrib import admin


class WydzialInlineForm(forms.ModelForm):
    class Meta:
        fields = ["nazwa", "skrot", "widoczny", "kolejnosc"]
        model = Wydzial
        widgets = {"kolejnosc": forms.HiddenInput}


class LiczbaNDlaUczelniInline(admin.TabularInline):
    model = LiczbaNDlaUczelni
    extra = 1

    class Meta:
        fields = ["dyscyplina_naukowa", "liczba_n"]


class WydzialInline(admin.TabularInline):
    model = Wydzial
    form = WydzialInlineForm
    extra = 0
    sortable_field_name = "kolejnosc"


class Ukryj_Status_KorektyInline(admin.StackedInline):
    model = Ukryj_Status_Korekty
    fields = [
        "status_korekty",
        "multiwyszukiwarka",
        "podglad",
        "raporty",
        "rankingi",
        "sloty",
        "api",
    ]
    extra = 0


class UczelniaAdmin(
    RestrictDeletionToAdministracjaGroupMixin,
    ZapiszZAdnotacjaMixin,
    BaseBppAdminMixin,
    admin.ModelAdmin,
):
    list_display = ["nazwa", "nazwa_dopelniacz_field", "skrot", "pbn_uid"]
    autocomplete_fields = ["pbn_uid", "obca_jednostka"]
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "nazwa",
                    "nazwa_dopelniacz_field",
                    "skrot",
                    "pbn_uid",
                    "pbn_id",
                    "favicon_ico",
                    "obca_jednostka",
                )
            },
        ),
        (
            "Opcje edycji",
            {
                "classes": ("grp-collapse grp-closed",),
                "fields": ("domyslnie_afiliuje",),
            },
        ),
        (
            "PBN API",
            {
                "classes": ("grp-collapse grp-open",),
                "fields": (
                    "pbn_integracja",
                    "pbn_aktualizuj_na_biezaco",
                    "pbn_api_root",
                    "pbn_app_name",
                    "pbn_app_token",
                    "pbn_api_user",
                    "pbn_api_kasuj_przed_wysylka",
                    "pbn_api_nie_wysylaj_prac_bez_pk",
                    "pbn_api_afiliacja_zawsze_na_uczelnie",
                ),
            },
        ),
        (
            "Strona wizualna",
            {
                "classes": ("grp-collapse grp-closed",),
                "fields": (
                    "logo_www",
                    "logo_svg",
                    "ilosc_jednostek_na_strone",
                    "pokazuj_tylko_jednostki_nadrzedne",
                    "ranking_autorow_rozbij_domyslnie",
                    "pokazuj_punktacje_wewnetrzna",
                    "pokazuj_index_copernicus",
                    "pokazuj_punktacja_snip",
                    "pokazuj_status_korekty",
                    "pokazuj_ranking_autorow",
                    "pokazuj_raport_autorow",
                    "pokazuj_raport_jednostek",
                    "pokazuj_raport_wydzialow",
                    "pokazuj_raport_uczelni",
                    "pokazuj_raport_dla_komisji_centralnej",
                    "pokazuj_praca_recenzowana",
                    "pokazuj_liczbe_cytowan_w_rankingu",
                    "pokazuj_liczbe_cytowan_na_stronie_autora",
                    "pokazuj_tabele_slotow_na_stronie_rekordu",
                    "pokazuj_raport_slotow_autor",
                    "pokazuj_raport_slotow_zerowy",
                    "pokazuj_raport_slotow_uczelnia",
                    "pokazuj_formularz_zglaszania_publikacji",
                    "wyszukiwanie_rekordy_na_strone_anonim",
                    "wyszukiwanie_rekordy_na_strone_zalogowany",
                    "sortuj_jednostki_alfabetycznie",
                    "metoda_do_roku_formularze",
                ),
            },
        ),
        (
            "Wydruki",
            {
                "classes": ("grp-collapse grp-closed",),
                "fields": (
                    "wydruk_logo",
                    "wydruk_logo_szerokosc",
                    "wydruk_parametry_zapytania",
                ),
            },
        ),
        (
            "Podpowiadanie dyscyplin",
            {
                "classes": ("grp-collapse grp-opened",),
                "fields": ("podpowiadaj_dyscypliny",),
            },
        ),
        (
            "Zgłaszanie publikacji",
            {
                "classes": ("grp-collapse grp-opened",),
                "fields": ("wymagaj_informacji_o_oplatach",),
            },
        ),
        ADNOTACJE_FIELDSET,
        (
            "Clarivate Analytics API",
            {
                "classes": ("grp-collapse grp-closed",),
                "fields": ("clarivate_username", "clarivate_password"),
            },
        ),
    )

    inlines = [
        WydzialInline,
        Ukryj_Status_KorektyInline,
        LiczbaNDlaUczelniInline,
    ]


admin.site.register(Uczelnia, UczelniaAdmin)



# Generated by Django 2.1.7 on 2019-07-01 07:14

from django.db import migrations, models
import django.db.models.deletion
import django.db.models.expressions


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('bpp', '0162_auto_20190629_1239'),
    ]

    operations = [
        migrations.CreateModel(
            name='CacheQueue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_on', models.DateTimeField()),
                ('last_updated_on', models.DateTimeField(auto_now=True)),
                ('completed_on', models.DateTimeField(blank=True, null=True)),
                ('started_on', models.DateTimeField(blank=True, null=True)),
                ('task_id', models.CharField(blank=True, max_length=36, null=True)),
                ('error', models.BooleanField(default=False)),
                ('info', models.TextField(blank=True, null=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
        ),
    ]
